# coding=utf-8
import os
import re
import json
import httpx
import datetime
from acfun import source
from acfun.page import *
from acfun.exceptions import *
from acfun.saver import AcSaver

__author__ = 'dolacmeo'


class AcClient:
    _client = None
    history = list()

    def __init__(self):
        self._client = httpx.Client(headers=source.header)

    @property
    def cookies(self):
        return self._client.cookies

    @property
    def stream(self):
        return self._client.stream

    def request(self, *args, **kwargs):
        return self._client.request(*args, **kwargs)

    def get(self, *args, **kwargs):
        url = kwargs.get('url', args[0])
        self.history.append(['GET', url, datetime.datetime.now()])
        return self._client.get(*args, **kwargs)

    def post(self, *args, **kwargs):
        url = kwargs.get('url', args[0])
        self.history.append(['POST', url, datetime.datetime.now()])
        return self._client.post(*args, **kwargs)

    def put(self, *args, **kwargs):
        url = kwargs.get('url', args[0])
        self.history.append(['PUT', url, datetime.datetime.now()])
        return self._client.put(*args, **kwargs)

    def patch(self, *args, **kwargs):
        url = kwargs.get('url', args[0])
        self.history.append(['PATCH', url, datetime.datetime.now()])
        return self._client.put(*args, **kwargs)

    def delete(self, *args, **kwargs):
        url = kwargs.get('url', args[0])
        self.history.append(['DELETE', url, datetime.datetime.now()])
        return self._client.put(*args, **kwargs)


class Acer:
    debug = False
    client = None
    req_count = 0
    BASE_PATH = os.getcwd()
    DOWNLOAD_PATH = None

    did = None
    uid = None
    username = None
    is_logined = False
    cookie = None
    personal = dict()
    tokens = dict()
    nav_data = dict()
    channel_data = source.ChannelList

    message = None
    moment = None
    favourite = None

    ac_index = AcIndex
    ac_channel = AcChannel
    ac_wen = AcWen
    ac_rank = AcRank
    ac_search = AcSearch
    ac_up = AcUp
    ac_article = AcArticle
    ac_video = AcVideo
    ac_bangumi = AcBangumi
    ac_album = AcAlbum
    ac_comment = AcComment
    ac_link = AcLink
    ac_image = AcImage

    def __init__(self, debug: bool = False):
        self.debug = debug
        self.client = AcClient() if debug else httpx.Client(headers=source.header)
        self.moment = AcMoment(self)
        self._get_nav()
        if self.check_online() is False:
            raise AcExploded("阿禅爆炸 (今天A站挂了吗？)")
        self._get_personal()

    def __repr__(self):
        show_name = self.username or ''
        return f"Acer(#{self.uid or 'UNKNOWN'} {show_name})"

    def AcIndex(self):
        return AcIndex(self)

    def AcDownload(self):
        return AcDownload(self)

    def AcChannel(self, cid):
        return AcChannel(self, cid, self.nav_data.get(int(cid)))

    def AcWen(self,
              realmIds: [list, None] = None,
              sortType: str = "createTime",
              timeRange: str = "all",
              onlyOriginal: bool = False,
              limit: int = 10):
        return AcWen(self, realmIds, sortType, timeRange, onlyOriginal, limit)

    def AcRank(self,
               cid: [int, None] = None,
               sub_cid: [int, None] = None,
               limit: int = 50,
               date_range: [str, None] = None):
        return AcRank(self, cid, sub_cid, limit, date_range)

    def AcSearch(self, keyword: [str, None] = None, s_type: [str, None] = None):
        return AcSearch(self, keyword, s_type)

    def AcUp(self, updata):
        return AcUp(self, updata)

    def AcLiveUp(self, uid: int, raw=None):
        return AcLiveUp(self, uid, raw)

    def AcArticle(self, ac_num, data=None):
        return AcArticle(self, ac_num, data)

    def AcVideo(self, ac_num, data=None):
        return AcVideo(self, ac_num, data)

    def AcBangumi(self, aa_num):
        return AcBangumi(self, aa_num)

    def AcComment(self, ac_num, sourceType: int, referer: [str, None] = None):
        return AcComment(self, ac_num, sourceType, referer)

    def AcAlbum(self, ac_num):
        return AcAlbum(self, ac_num)

    def AcLive(self):
        return AcLive(self)

    def AcLink(self, url, title, container=None):
        return AcLink(self, url, title, container)

    def AcImage(self, src, url=None, name=None, container=None):
        return AcImage(self, src, url, name, container)

    def AcSaver(self, dest_path: [str, None] = None):
        return AcSaver(self, None, dest_path)

    def get(self, url_str: str):
        for link_name in ['video', 'article', 'album', 'bangumi', 'up', 'moment', 'live', 'share']:
            if url_str.startswith(source.routes[link_name]):
                ends = url_str[len(source.routes[link_name]):]
                if link_name == 'up':
                    return self.AcUp({'userId': ends})
                elif link_name == 'moment':
                    return self.moment.get(ends)
                elif link_name == 'share':
                    return self.AcVideo(ends)
                elif link_name == 'live':
                    return self.AcLiveUp(int(ends))
                return getattr(self, f"Ac{link_name.capitalize()}")(ends)
        channel_rex = re.compile(f"^{source.routes['index']}/v/list(\d+)/index.htm$").findall(url_str)
        if channel_rex:
            return self.AcChannel(channel_rex[0])
        return None

    def check_online(self):
        req = self.client.get(source.routes['ico'], timeout=10)
        return req.status_code == 200

    def _get_nav(self):
        data = self.client.get(source.apis['nav']).json().get("data", [])
        for i in data:
            if i['cid'] != 0:
                self.nav_data.update({i['cid']: {x: i[x] for x in i if x != 'children'}})
            for j in i['children']:
                if j['cid'] != 0:
                    self.nav_data.update({j['cid']: {y: j[y] for y in j if y != 'children'}})

    def _get_personal(self):
        live_page_req = self.client.get(source.routes['live_index'])
        assert live_page_req.status_code // 100 == 2
        self.did = live_page_req.cookies.get('_did')
        if self.is_logined:
            api_req = self.client.post(source.apis['token'], data={"sid": "acfun.midground.api"})
            api_data = api_req.json()
            assert api_data.get('result') == 0
            self.tokens = {
                "ssecurity": api_data.get("ssecurity", ''),
                "api_st": api_data.get("acfun.midground.api_st", ''),
                "api_at": api_data.get("acfun.midground.api.at", ''),
            }
            info_req = self.client.get(source.apis['personalInfo'])
            info_data = info_req.json()
            assert info_data.get('result') == 0
            self.personal = info_data.get('info', {})
            self.uid = self.personal.get('userId')
            self.username = self.personal.get('userName')
            self.message = AcMessage(self)
            self.favourite = MyFavourite(self)
        else:
            api_req = self.client.post(source.apis['token_visitor'], data={"sid": "acfun.api.visitor"})
            api_data = api_req.json()
            assert api_data.get('result') == 0
            self.tokens = {
                "ssecurity": api_data.get("acSecurity", ''),
                "visitor_st": api_data.get("acfun.api.visitor_st", ''),
            }
            self.uid = api_data.get("userId")

    def update_token(self, data: dict):
        if self.is_logined:
            data.update({"acfun.midground.api_st": self.tokens['api_st']})
        else:
            data.update({"acfun.api.visitor_st": self.tokens['visitor_st']})
        return data

    def keys(self):
        return self.personal.keys()

    def loading(self, username):
        cookie_data = open(f'{username}.cookies', 'rb').read()
        cookie = B64s(cookie_data, len(username)).b64decode()
        self.cookie = json.loads(cookie.decode())
        self.client.cookies.update(self.cookie)
        self.is_logined = True
        self._get_personal()

    def login(self, username, password, key=None, captcha=None):
        form_data = {
            "username": username,
            "password": password,
            "key": key or "",
            "captcha": captcha or ""
        }
        api_req = self.client.post(source.apis['login'], data=form_data)
        result = api_req.json()
        self.is_logined = result.get('result', 1) == 0
        if self.is_logined is True:
            self._get_personal()
            cookie = json.dumps(dict(self.client.cookies.items()), separators=(',', ':'))
            cookie = B64s(cookie.encode(), len(username)).b64encode()
            with open(f'{username}.cookies', 'wb') as f:
                f.write(cookie)
        else:
            print(result)
            raise NotInCar('登录没成功，你到底是什么搞错了啊！')
        return self.is_logined

    def logout(self):
        self.client.get(source.apis['logout'])
        self.client = httpx.Client(headers=source.header)
        self.is_logined = False
        self.personal = dict()
        return True

    @need_login
    def update_signature(self, text: str):
        api_req = self.client.post(source.apis['updateSignature'], data={'signature': text},
                                   headers={'referer': 'https://www.acfun.cn/member/setting?tab=info'})
        return api_req.json().get('result') == 0

    @need_login
    def signin(self):
        api_req = self.client.get(source.apis['signIn'])
        api_data = api_req.json()
        print(api_data.get('msg'))
        return api_data.get('result') == 0

    @need_login
    def acoin(self):
        req = self.client.get(source.apis['acoinBalance'])
        data = req.json()
        if data.get('result') == 0:
            return data.get('data')
        return None

    @need_login
    def like(self, obj_id: str, objectType: int):
        form_data = {
            "kpn": "ACFUN_APP",
            "kpf": "PC_WEB",
            "subBiz": "mainApp",
            "interactType": 1,
            "objectType": objectType,
            "objectId": obj_id,
            "userId": self.uid,
        }
        form_data = self.update_token(form_data)
        req = self.client.post(source.apis['like_add'], data=form_data)
        return req.json().get('result') == 1

    @need_login
    def like_cancel(self, obj_id: str, objectType: int):
        form_data = {
            "kpn": "ACFUN_APP",
            "kpf": "PC_WEB",
            "subBiz": "mainApp",
            "interactType": 1,
            "objectType": objectType,
            "objectId": obj_id,
            "userId": self.uid,
        }
        form_data = self.update_token(form_data)
        req = self.client.post(source.apis['like_delete'], data=form_data)
        return req.json().get('result') == 1

    @need_login
    def throw_banana(self, ac_num, rt: int, count: int):
        api_req = self.client.post(source.apis['throw_banana'], data={
            "resourceId": ac_num,
            "count": 1 if 1 > count > 5 else count,
            "resourceType": rt
        }, headers={'referer': source.routes['index']})
        return api_req.json().get('result') == 0

    @need_login
    def follow_groups(self):
        api_req = self.client.get(source.apis['follow_groups'])
        api_data = api_req.json()
        if api_data.get('result') == 0:
            return api_data.get('groupList', [])
        return None

    def _follow_group_action(self, form_data: dict):
        api_req = self.client.post(source.apis['follow_group'], data=form_data)
        return api_req.json().get('result') == 0

    @need_login
    def follow_group_add(self, name: str):
        form_data = {"action": 4, "groupName": name}
        return self._follow_group_action(form_data)

    @need_login
    def follow_group_rename(self, gid: [int, str], name: str):
        form_data = {"action": 6, "groupId": gid, "groupName": name}
        return self._follow_group_action(form_data)

    @need_login
    def follow_group_remove(self, gid: [int, str]):
        form_data = {"action": 5, "groupId": gid}
        return self._follow_group_action(form_data)

    @need_login
    def follow_add(self, uid, attention: [bool, None] = None):
        form_data = {"toUserId": uid, "action": 1}
        if attention is True:
            form_data['action'] = 14
        elif attention is False:
            form_data['action'] = 15
        if form_data['action'] == 1:
            form_data['groupId'] = 0
        api_req = self.client.post(source.apis['follow'], data=form_data)
        return api_req.json().get('result') == 0

    @need_login
    def follow_remove(self, uid):
        form_data = {"toUserId": uid, "action": 2}
        api_req = self.client.post(source.apis['follow'], data=form_data)
        return api_req.json().get('result') == 0

    @need_login
    def history(self, page: int = 1, limit: int = 10, obj: bool = False):
        # 观看历史
        form_data = {"pageNo": page, "pageSize": limit, "resourceTypes": ''}
        api_req = self.client.post(source.apis['history'], data=form_data)
        api_data = api_req.json()
        if api_data.get('result') != 0:
            return None
        histories = api_data.get('histories', [])
        if obj is True:
            objs = list()
            for x in histories:
                if x.get('resourceType') == 1:
                    objs.append(self.AcBangumi(x.get('resourceId')))
                elif x.get('resourceType') == 2:
                    objs.append(self.AcVideo(x.get('resourceId'), x))
                elif x.get('resourceType') == 3:
                    objs.append(self.AcArticle(x.get('resourceId'), x))
            return objs
        return histories

    @need_login
    def my_fans(self, page: int = 1, limit: int = 10, obj: bool = False):
        # 粉丝列表
        form_data = {"page": page, "count": limit, "action": 8}
        api_req = self.client.post(source.apis['follow_fans'], data=form_data)
        api_data = api_req.json()
        if api_data.get('result') != 0:
            return None
        fans = api_data.get('friendList', [])
        if obj is True:
            return [self.AcUp(x) for x in fans]
        return fans

    def _get_my_posted(self,
                       rtype: int,
                       page: int = 1,
                       status: str = 'all',
                       sortby: str = 'recently',
                       keyword: [str, None] = None):
        # 我的文章
        status_list = {
            'all': 0,  # 全部
            'passed': 1,  # 已通过
            'posting': 2,  # 发布中
            'returned': 3,  # 已退回
        }
        sort_list = {
            'viwed': 1,  # 最多阅读
            'banana': 2,  # 最多香蕉
            'recently': 3,  # 最新发布
        }
        form_data = {
            "pcursor": 0 if (page - 1) < 0 else (page - 1),
            "resourceType": rtype,  # 2 视频 3 文章
            "authorId": self.uid,
            "sortType": sort_list.get(sortby, 3),
            "status": status_list.get(status, 0),
            "keyword": keyword,
        }
        api_req = self.client.post(source.apis['member_posted'], data=form_data)
        api_data = api_req.json()
        if api_data.get('result') == 0:
            return api_data.get('feed', [])
        return None

    @need_login
    def my_articles(self, page: int = 1, status: str = 'all', sortby: str = 'recently', keyword: [str, None] = None):
        return self._get_my_posted(3, page, status, sortby, keyword)

    @need_login
    def my_videos(self, page: int = 1, status: str = 'all', sortby: str = 'recently', keyword: [str, None] = None):
        return self._get_my_posted(2, page, status, sortby, keyword)

    @need_login
    def my_danmaku(self):
        # 弹幕管理
        pass

    @need_login
    def data_center(self, days: int = 1):
        api_req = self.client.post(source.apis['dataCenter_overview'], data={'days': 1 if days < 1 else days})
        api_data = api_req.json()
        if api_data.get('result') == 0:
            return api_data
        return None

    @need_login
    def data_center_detail(self, rtype: str, days: int = 1, sortby: str = 'viewCount'):
        rtypes = {'video': 2, 'article': 3, 'live': None}
        assert rtype in rtypes.keys()
        sort_list = {
            "viewCount": "阅读量",
            "commentCount": "评论量",
            "stowCount": "收藏量",
            "shareCount": "分享量",
            "bananaCount": "投蕉量"
        }
        assert sortby in sort_list
        form_data = {'days': 1 if days < 1 else days}
        if rtype == 'live':
            api_req = self.client.post(source.apis['dataCenter_live'], data=form_data)
        else:
            form_data.update({"resourceType": rtypes.get(rtype), "orderBy": sortby})
            api_req = self.client.post(source.apis['dataCenter_content'], data=form_data)
        api_data = api_req.json()
        if api_data.get('result') == 0:
            return api_data
        return None

    @need_login
    def get_live_config(self):
        param = {
            "kpn": "ACFUN_APP",
            "kpf": "PC_WEB",
            "userId": self.uid,
            "subBiz": "mainApp"
        }
        param = self.update_token(param)
        api_req = self.client.post(source.apis['live_obs_config'], params=param,
                                   headers={'referer': f"{source.scheme}://{source.domains['user']}"})
        return api_req.json()

    @need_login
    def post_article(self):
        # 发文章
        pass

    @need_login
    def post_video(self):
        # 发视频
        pass


class MyFavourite:
    folders = list()
    default_fid = None

    def __init__(self, acer):
        self.acer = acer
        self.video_groups()

    def add(self, obj_id: str, rType: int, fids: [str, None] = None):
        form_data = {"resourceId": obj_id, "resourceType": rType}
        if fids is not None or rType == 9:
            form_data['addFolderIds'] = str(fids or self.default_fid)
        req = self.acer.client.post(source.apis['favorite_add'], data=form_data,
                                    headers={"referer": source.routes['index']})
        return req.json().get('result') == 0

    def cancel(self, obj_id: str, rType: int, fids: [str, None] = None):
        form_data = {"resourceId": obj_id, "resourceType": rType}
        if fids is not None or rType == 9:
            form_data['delFolderIds'] = fids or self.default_fid
        req = self.acer.client.post(source.apis['favorite_remove'], data=form_data,
                                    headers={"referer": source.routes['index']})
        return req.json().get('result') == 0

    def video_groups(self):
        api_req = self.acer.client.get(source.apis['video_favorite_list'])
        api_data = api_req.json()
        if api_data.get('result') == 0:
            self.folders = api_data.get('dataList', [])
            self.default_fid = self.folders[0].get('folderId')
            return self.folders
        return None

    def video_group_add(self, name: str):
        form_data = {"name": name}
        api_req = self.acer.client.post(source.apis['video_favorite_group_add'], data=form_data,
                                        headers={"referer": source.routes['index']})
        return api_req.json().get('result') == 0

    def video_group_rename(self, fid: [int, str], name: str):
        form_data = {"folderId": fid, "name": name}
        api_req = self.acer.client.post(source.apis['video_favorite_group_update'], data=form_data,
                                        headers={"referer": source.routes['index']})
        return api_req.json().get('result') == 0

    def video_group_remove(self, fid: [int, str]):
        form_data = {"folderId": fid}
        api_req = self.acer.client.post(source.apis['video_favorite_group_delete'], data=form_data,
                                        headers={"referer": source.routes['index']})
        return api_req.json().get('result') == 0

    def video_list(self, fid: [int, str], page: int = 1, limit: int = 10):
        form_data = {"folderId": fid, "page": page, "perpage": limit}
        api_req = self.acer.client.post(source.apis['favorite_video'], data=form_data)
        data = api_req.json()
        if data.get('result') == 0:
            return data.get('favoriteList', [])
        return None

    def article_list(self, page: int = 1, limit: int = 10):
        form_data = {"page": page, "perpage": limit}
        api_req = self.acer.client.post(source.apis['favorite_article'], data=form_data)
        data = api_req.json()
        if data.get('result') == 0:
            return data.get('favoriteList', [])
        return None

    def bangumi_list(self, page: int = 1, limit: int = 10):
        param = {"page": page, "perpage": limit}
        api_req = self.acer.client.get(source.apis['favorite_article'], params=param)
        data = api_req.json()
        if data.get('result') == 0:
            return data.get('favoriteList', [])
        return None

    def album_list(self, page: int = 1, limit: int = 10):
        form_data = {"page": page, "perpage": limit}
        api_req = self.acer.client.post(source.apis['favorite_album'], data=form_data)
        data = api_req.json()
        if data.get('result') == 0:
            return data.get('favoriteList', [])
        return None




# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: CommonStateSignalPKReady.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


import PkPlayerInfo_pb2 as PkPlayerInfo__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1e\x43ommonStateSignalPKReady.proto\x12\nAcFunDanmu\x1a\x12PkPlayerInfo.proto\"J\n\x18\x43ommonStateSignalPKReady\x12\t\n\x01\x61\x18\x01 \x01(\t\x12#\n\x01\x62\x18\x02 \x03(\x0b\x32\x18.AcFunDanmu.PkPlayerInfob\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'CommonStateSignalPKReady_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _COMMONSTATESIGNALPKREADY._serialized_start=66
  _COMMONSTATESIGNALPKREADY._serialized_end=140
# @@protoc_insertion_point(module_scope)



# coding=utf-8
import os
import json
import time
import random
import base64
import psutil
import threading
import websocket
import subprocess
from acfun.source import routes, apis, websocket_links
from acfun.protos import AcProtos
from rich import print, console


__author__ = 'dolacmeo'

# https://github.com/wpscott/AcFunDanmaku/blob/master/AcFunDanmu/README.md
# https://github.com/wpscott/AcFunDanmaku/blob/master/AcFunDanmu/data.md
# https://github.com/orzogc/acfundanmu/blob/master/proto.go
# https://developers.google.com/protocol-buffers/docs/pythontutorial
# https://websocket-client.readthedocs.io/en/latest/getting_started.html

# https://protogen.marcgravell.com/decode

# websocket 异步线程 同步化 + 背景进程
# https://stackoverflow.com/questions/51197063/python-websockets-in-synchronous-program
# 构建多线程异步 websocket
# 在同步返回结果前存留seqId为标识的线程
# 根据异步返回结果匹配seqId线程，并将结果返回给等待中的线程，异步变为同步
# 主要进程如下，需要线程守护：
# 1.背景线程持续接收服务器返回信息
# 2.keep alive 保持长连接
# 3.等待接收新的指令
# 进程初始需要执行初始化：注册
# 将注册后一系列过程同步，并保持必要信息


def uint8_payload_to_base64(data: dict):
    """
    用于反解网页中等待encode的payload
    进入页面: https://message.acfun.cn/im
    调试js  : https://static.yximgs.com/udata/pkg/acfun-im/ImSdk.b0aeed.js
    进入页面: https://live.acfun.cn/live/
    设置断点: 9145  => e.payloadData
    调试js  : https://ali-imgs.acfun.cn/kos/nlav10360/static/js/3.867c7c46.js
    设置断点: 40910 => t
    return: base64encoded ==> https://protogen.marcgravell.com/decode
    """
    b_str = b''
    for x in range(len(data.keys())):
        b_str += bytes([data[str(x)]])
    return base64.standard_b64encode(b_str)


def unix2string(t: (int, float, str), f: str = "%H:%M:%S"):
    if len(str(t)) > 10:
        t = int(str(t)[:10])
    return time.strftime(f, time.localtime(t))


def ac_live_room_reader(data: list, gift_data: [dict, None] = None, msg_bans: [list, None] = None):
    message_types = {
        "ZtLiveScActionSignal": "(粉丝互动)",
        "ZtLiveScStateSignal": "(数据更新)",
        "ZtLiveScNotifySignal": "(房管来啦)",
        "ZtLiveScStatusChanged": "(状态变更)",
        "ZtLiveScTicketInvalid": "(连接失效)[需要重连]",
    }
    msg_bans = [] if msg_bans is None else msg_bans
    signal_types = {
        "CommonActionSignalComment": "[送出弹幕]",
        "CommonActionSignalLike": "[爱心点赞]",
        "CommonActionSignalUserEnterRoom": "[进入房间]",
        "CommonActionSignalUserFollowAuthor": "[关注主播]",
        "AcfunActionSignalThrowBanana": "[投喂香蕉]",
        "CommonActionSignalGift": "[送出礼物]",
        "CommonActionSignalRichText": "[高级弹幕]",
        "AcfunActionSignalJoinClub": "[加守护团]",
        "AcfunStateSignalDisplayInfo": "[香蕉总数]",
        "CommonStateSignalDisplayInfo": "[在线人数][点赞数量]",
        "CommonStateSignalTopUsers": "[前三粉丝]",
        "CommonStateSignalRecentComment": "[近期弹幕]",
        "CommonStateSignalChatCall": "[连麦被叫呼叫]",
        "CommonStateSignalChatAccept": "[连麦被叫接受]",
        "CommonStateSignalChatReady": "[连麦被叫等待]",
        "CommonStateSignalChatEnd": "[连麦被叫结束]",
        "CommonStateSignalCurrentRedpackList": "[红包榜单]",
        "CommonStateSignalAuthorChatCall": "[连麦主叫呼叫]",
        "CommonStateSignalAuthorChatAccept": "[连麦主叫接受]",
        "CommonStateSignalAuthorChatReady": "[连麦主叫等待]",
        "CommonStateSignalAuthorChatEnd": "[连麦主叫结束]",
        "CommonStateSignalAuthorChatChangeSoundConfig": "[连麦主叫导播]",
        "CommonStateSignalPKAccept": "[连麦挑战接受]",
        "CommonStateSignalPKInvitation": "[连麦挑战邀请]",
        "CommonStateSignalPKReady": "[连麦挑战等待]",
        "CommonStateSignalPKSoundConfigChanged": "[连麦挑战导播]",
        "CommonStateSignalPkEnd": "[连麦挑战结束]",
        "CommonStateSignalPkStatistic": "[连麦挑战统计]",
        "CommonStateSignalWishSheetCurrentState": "[愿望列表状态]",
        "CommonNotifySignalKickedOut": "[踢出房间]",
        "CommonNotifySignalViolationAlert": "[违规警告]",
        "CommonNotifySignalLiveManagerState": "[房管状态]",
    }

    def user_info(payload_item):
        payload = payload_item['userInfo']
        base = f"<{payload['userId']}@{payload['nickname']}>"
        if 'badge' in payload:
            badge = json.loads(payload['badge']).get('medalInfo', {})
            base += f"『{badge['clubName']}|lv{badge['level']}』"
        return base

    messages = list()
    for item in data:
        signal_path = item['signal']
        if item['signal'].count(".") == 0:
            signal_path += "."
        msg_type, signal_name = signal_path.split(".")
        if msg_type in msg_bans:
            continue
        words = list()
        payload = item.get('payload')
        # 消息类型
        words.append(message_types.get(msg_type, "(????????)"))
        # 信号类型
        if signal_name:
            words.append(signal_types.get(signal_name, "[????????]"))
        # 内容信息
        if signal_name == "CommonActionSignalComment":
            words = list()
            for fans in payload:
                users = list()
                # 消息类型
                users.append(message_types.get(msg_type, "(????????)"))
                # 信号类型
                if signal_name:
                    users.append(signal_types.get(signal_name, "[????????]"))
                # 内容信息
                user = user_info(fans)
                content = fans['content']
                send_time = unix2string(fans['sendTimeMs'])
                users.append(f"{{{send_time}}} \r\n{user} 💬{content} \r\n")
                words.append("".join(users))
        elif signal_name == "CommonActionSignalLike":
            words = list()
            for fans in payload:
                users = list()
                # 消息类型
                users.append(message_types.get(msg_type, "(????????)"))
                # 信号类型
                if signal_name:
                    users.append(signal_types.get(signal_name, "[????????]"))
                # 内容信息
                user = user_info(fans)
                send_time = unix2string(fans['sendTimeMs'])
                users.append(f"{{{send_time}}} \r\n{user} 点赞了❤ \r\n")
                words.append("".join(users))
        elif signal_name == "CommonActionSignalUserEnterRoom":
            words = list()
            for newbee in payload:
                new_user = list()
                # 消息类型
                new_user.append(message_types.get(msg_type, "(????????)"))
                # 信号类型
                if signal_name:
                    new_user.append(signal_types.get(signal_name, "[????????]"))
                # 内容信息
                user = user_info(newbee)
                send_time = unix2string(int(newbee['sendTimeMs']))
                new_user.append(f"{{{send_time}}} \r\n{user} 进入直播间👤 \r\n")
                words.append("".join(new_user))
        elif signal_name == "CommonActionSignalUserFollowAuthor":
            words = list()
            for newbee in payload:
                new_user = list()
                # 消息类型
                new_user.append(message_types.get(msg_type, "(????????)"))
                # 信号类型
                if signal_name:
                    new_user.append(signal_types.get(signal_name, "[????????]"))
                # 内容信息
                user = user_info(newbee)
                send_time = unix2string(int(newbee['sendTimeMs']))
                words.append(f"{{{send_time}}} \r\n{user} 关注了主播👀 \r\n")
                words.append("".join(new_user))
        elif signal_name == "AcfunActionSignalThrowBanana":
            user = user_info(payload)
            send_time = unix2string(int(payload['sendTimeMs']))
            words.append(f"{{{send_time}}}{user}")
        elif signal_name == "CommonActionSignalGift":
            words = list()
            for fans in payload:
                users = list()
                # 消息类型
                users.append(message_types.get(msg_type, "(????????)"))
                # 信号类型
                if signal_name:
                    users.append(signal_types.get(signal_name, "[????????]"))
                # 内容信息
                user = user_info(fans)
                if gift_data is None:
                    gift = f"送出{fans['batchSize']}个🎁[{fans['giftId']}]"
                else:
                    gift = f"送出{fans['batchSize']}个🎁[{gift_data[str(fans['giftId'])]['giftName']}]"
                if fans['comboCount'] > 1:
                    gift += f" 连击{fans['comboCount']}"
                send_time = unix2string(fans['sendTimeMs'])
                words.append(f"{{{send_time}}} \r\n{user} {gift} \r\n")
                words.append("".join(users))
        elif signal_name == "CommonActionSignalRichText":  # 高级弹幕
            pass
        elif signal_name == "AcfunActionSignalJoinClub":
            words = list()
            for fans in payload:
                users = list()
                # 消息类型
                users.append(message_types.get(msg_type, "(????????)"))
                # 信号类型
                if signal_name:
                    users.append(signal_types.get(signal_name, "[????????]"))
                # 内容信息
                user = user_info(fans)
                send_time = unix2string(fans['sendTimeMs'])
                words.append(f"{{{send_time}}} \r\n{user} 加入守护团 \r\n")
                words.append("".join(users))
        elif signal_name == "AcfunStateSignalDisplayInfo":
            words.append(f"🍌x{payload['bananaCount']}")
        elif signal_name == "CommonStateSignalDisplayInfo":
            if 'watchingCount' in payload:
                words.append(f" 👤x{payload['watchingCount']}")
            if 'likeCount' in payload:
                words.append(f" ❤x{payload['likeCount']}")
        elif signal_name == "CommonStateSignalTopUsers":
            tops = [user_info(u) for u in payload['user']]
            words.append(f"\r\n🥇{tops[0]}")
            words.append(f"\r\n🥈{tops[1]}")
            words.append(f"\r\n🥉{tops[2]}")
        elif signal_name == "CommonStateSignalRecentComment":
            words = list()
            for comment in payload['comment']:
                his_words = list()
                # 消息类型
                his_words.append(message_types.get(msg_type, "(????????)"))
                # 信号类型
                if signal_name:
                    his_words.append(signal_types.get(signal_name, "[????????]"))
                # 内容信息
                user = user_info(comment)
                content = comment['content']
                send_time = unix2string(int(comment['sendTimeMs']))
                his_words.append(f"{{{send_time}}} \r\n{user} 💬{content}")
                full_comment = "".join(his_words) + "\r\n"
                words.append(full_comment)
        elif signal_name == "CommonNotifySignalKickedOut":
            words.append(f"{payload['reason']}")
        elif signal_name == "CommonNotifySignalViolationAlert":
            words.append(f"{payload['violationContent']}")
        elif signal_name == "CommonNotifySignalLiveManagerState":
            # MANAGER_STATE_UNKNOWN = 0
            # MANAGER_ADDED = 1
            # MANAGER_REMOVED = 2
            # IS_MANAGER = 3
            words.append(f"{payload['state']}")
        this_words = "".join(words)
        if this_words.endswith("\r\n"):
            this_words = this_words[:-2]
        messages.append(this_words)
    return messages


class AcWebSocket:
    appId = 0
    instanceId = 0
    ws_link = None
    config = None
    _main_thread = None
    tasks = dict()
    commands = dict()
    unread = []
    is_register_done = False
    live_room = None
    player_config = None
    _console = console.Console(width=100)
    live_room_msg_bans = []
    live_room_gift = None
    live_obj = None
    _live_player = None
    is_close = True

    def __init__(self, acer):
        self.acer = acer
        # websocket.enableTrace(True)
        self.ws_link = random.choice(websocket_links)
        self.protos = AcProtos(self)
        self.ws = websocket.WebSocketApp(
            url=self.ws_link,
            on_open=self._register,
            on_message=self._message,
            on_error=self._on_error,
            on_close=self._on_close,
            on_ping=self._keep_alive_request,
            on_pong=self._keep_alive_response,
        )
        self.listenner = dict()

    def run(self):
        def _run():
            self.ws.run_forever(
                ping_interval=10, ping_timeout=5,
                skip_utf8_validation=True,
                origin="live.acfun.cn",
            )
        self._main_thread = threading.Thread(target=_run)
        self._main_thread.start()
        self.is_close = False
        return self

    def add_task(self, seqId: int, command, content):
        if self.is_close is True:
            return False
        if f"{seqId}" not in self.tasks:
            self.tasks[f"{seqId}"] = dict()
        self.tasks[f"{seqId}"]["send"] = {"command": command, "content": content, "time": time.time()}
        if command not in self.commands:
            self.commands[command] = []
        self.commands[command].append({"seqId": f"{seqId}", "way": "send", "time": time.time()})
        # print(f"send: [{seqId}] {command}")
        self.ws.send(content)

    def task(self, seqId: int, command, content):
        self.add_task(seqId, command, content)
        # if f"{seqId}" not in self.listenner:
        #     self.listenner[f"{seqId}"] = None

    def ans_task(self, seqId: int, command, result):
        if f"{seqId}" not in self.tasks:
            self.tasks[f"{seqId}"] = {}
        self.tasks[f"{seqId}"]["recv"] = {"command": command, "content": result, "time": time.time()}
        if command not in self.commands:
            self.commands[command] = []
        self.commands[command].append({"seqId": f"{seqId}", "way": "recv", "time": time.time()})
        # if f"{seqId}" in self.listenner:
        #     self.listenner[f"{seqId}"] = result
        self.unread.append(f"{seqId}.recv")
        if command == 'Basic.Register':
            self.task(*self.protos.ClientConfigGet_Request())
            print(f"did       : {self.acer.did}")
            print(f"userId    : {self.acer.uid}")
            print(f"ssecurity : {self.acer.tokens['ssecurity']}")
            print(f"sessKey   : {self.acer.tokens['sessKey']}")
            self.is_register_done = True
            print(">>>>>>>> AcWebsocket Registed<<<<<<<<<")
        elif command == 'Basic.ClientConfigGet':
            self.task(*self.protos.KeepAlive_Request())
            self.protos.client_config = result
            print(">>>>>>>> AcWebsocket  Ready  <<<<<<<<<")
        elif command == 'LiveCmd.ZtLiveCsEnterRoomAck':
            self.live_room = self.protos.live_room
            self.live_room_gift = self.live_obj.gift_list()
            self.task(*self.protos.ZtLiveCsHeartbeat_Request())
            self.task(*self.protos.ZtLiveInteractiveMessage_Request())
            print(">>>>>>>> AcWebsocket EnterRoom <<<<<<<<")
            live_data = json.loads(self.live_room.get('videoPlayRes', "")).get('liveAdaptiveManifest', [])[0]
            live_adapt = live_data.get('adaptationSet', {}).get('representation', {})
            if self.player_config is None:
                print(f"未设置PotPlayer 请使用串流地址 请自行播放 \r\n {live_adapt[2]['url']}")
            else:
                create_time = self.live_room.get('liveStartTime', 0) // 1000
                start_time = time.strftime('%Y-%m-%d %H:%M:%S', time.localtime(create_time))
                live_up_name = self.live_obj.username
                live_type = self.live_obj.raw.get("type", {})
                live_title = " ".join([
                    f"AcLive(#{self.live_obj.uid} @{live_up_name}| {live_type['categoryName']}>>{live_type['name']})",
                    self.live_room['caption'], f"🎬 {start_time}"
                ])
                potplayer = self.player_config['player_path']
                print(f"[{potplayer}] 开始播放......\r\n {live_title}")
                live_obs_stream = live_adapt[self.player_config['quality']]['url']
                cmd_list = [potplayer, live_obs_stream, "/title", f'"{live_title}"']
                self._live_player = subprocess.Popen(cmd_list, creationflags=subprocess.CREATE_NO_WINDOW)
        if command.startswith("LivePush.") and result:
            msg = ac_live_room_reader(result, self.live_room_gift, self.live_room_msg_bans)
            for n in msg:
                self._console.print(n)

    def _register(self, ws):
        print(">>>>>>> AcWebsocket Connecting<<<<<<<<")
        self.task(*self.protos.BasicRegister_Request())

    def _message(self, ws, message):
        self.ans_task(*self.protos.decode(message))

    def _keep_alive_request(self, ws, message):
        self.add_task(*self.protos.BasicPing_Request())

    def _keep_alive_response(self, ws, message):
        if self.is_register_done:
            self.add_task(*self.protos.KeepAlive_Request())

    def _on_close(self, ws, close_status_code, close_msg):
        # Because on_close was triggered, we know the opcode = 8
        if close_status_code or close_msg:
            print("on_close args:")
            print(f"  close status code: {close_status_code}")
            print(f"  close message    : {close_msg}")
        print(">>>>>>>> AcWebsocket  CLOSED <<<<<<<<<")

    def _on_error(self, ws, e):
        print("error: ", e)
        self.close()

    def close(self):
        self.add_task(*self.protos.Unregister_Request())
        self.is_close = True
        if self._live_player is not None:
            parent_proc = psutil.Process(self._live_player.pid)
            for child_proc in parent_proc.children(recursive=True):
                child_proc.kill()
            parent_proc.kill()
        self.ws.close()

    def restart(self):
        print(">>>>>>>> AcWebsocket  RESTART <<<<<<<<<")
        self.close()
        self.run()

    def im_get_sessions(self):
        message = self.protos.MessageSession_Request()
        return self.task(*message)

    def im_session_start(self, uid: int):
        message = self.protos.SessionCreate_Request(uid)
        return self.task(*message)

    def im_session_close(self, uid: int):
        message = self.protos.SessionRemove_Request(uid)
        return self.task(*message)

    def im_pull_message(self, uid: int, minSeq: int, maxSeq: int, count: int = 10):
        message = self.protos.MessagePullOld_Request(uid, minSeq, maxSeq, count)
        return self.task(*message)

    def im_send(self, uid: int, content: str):
        message = self.protos.MessageContent_Request(uid, content)
        return self.task(*message)

    def im_send_image(self, uid: int, image_data: bytes):
        message = self.protos.MessageImage_Request(uid, image_data)
        return self.task(*message)

    def live_enter_room(self, uid: int, room_bans: [list, None] = None,
                        potplayer: [str, None] = None, quality: int = 1):
        if isinstance(potplayer, str) and os.path.isfile(potplayer):
            self.player_config = {"player_path": potplayer, "quality": quality}
        self.live_room_msg_bans = [] if room_bans is None else room_bans
        self.live_obj = self.acer.AcLive().get(uid)
        cmd = self.protos.ZtLiveCsEnterRoom_Request(uid)
        return self.task(*cmd)




# coding=utf-8
import re
import os
import time
import math
import json
import httpx
import random
import base64
import shutil
import cssutils
import filetype
from uuid import uuid4
from urllib.parse import urlparse
from urllib.parse import urlencode
from bs4 import BeautifulSoup as Bs
from bs4.element import Tag
from datetime import timedelta
from alive_progress import alive_bar
from acfun.source import scheme, domains, routes, apis, pagelets, pagelets_big, pagelets_normal, videoQualitiesRefer
from acfun.exceptions import *

__author__ = 'dolacmeo'


class B64s:
    STANDARD = b'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/'
    EN_TRANS = STANDARD
    DE_TRANS = STANDARD

    def __init__(self, s: [bytes, bytearray], n: [int, None] = None):
        self.raw = s
        if isinstance(n, int):
            n = n % 64
            new = self.STANDARD[n:] + self.STANDARD[:n]
            self.EN_TRANS = bytes.maketrans(self.STANDARD, new)
            self.DE_TRANS = bytes.maketrans(new, self.STANDARD)

    def b64encode(self):
        return base64.standard_b64encode(self.raw).translate(self.EN_TRANS)

    def b64decode(self):
        return base64.b64decode(self.raw.translate(self.DE_TRANS))


def danmaku2ass(client, folder_path: str, filenameId: str, vq: str = "720p", fontsize: int = 40):
    """
    https://github.com/niuchaobo/acfun-helper/blob/master/src/fg/modules/danmaku.js
    基础代码复刻自acfun助手中弹幕相关处理
    关于解决原代码中的弹幕重叠问题：
        0. 原弹幕数据要按时间进行排序
        1. 记录每条弹幕通道最后截止位置
        2. 如果同期所有通道已满，则减少弹幕停留时间(加速通过)

    :param client: acer.client
    :param folder_path: source path
    :param filenameId: ac_num
    :param vq: VideoQuality
    :return: ass file path
    :param fontsize: num px
    """

    # 检查路径
    assert os.path.isdir(folder_path) is True
    folder_name = os.path.basename(folder_path)
    video_data_path = os.path.join(folder_path, 'data', f"{filenameId}.json")
    danmaku_data_path = os.path.join(folder_path, 'data', f"{filenameId}.danmaku.json")
    assert all([os.path.isfile(video_data_path), os.path.isfile(danmaku_data_path)]) is True
    video_data = json.load(open(video_data_path, 'rb'))
    danmaku_data = json.load(open(danmaku_data_path, 'rb'))
    danmuMotionList = []
    if len(danmaku_data) == 0:
        return None

    thisVideoInfo = videoQualitiesRefer[vq]
    thisVideoWidth = thisVideoInfo['width']
    thisVideoHeight = thisVideoInfo['height']
    thisDuration = 10
    channelNum = math.floor(thisVideoWidth / fontsize)
    scriptInfo = "\n".join([
        "[Script Info]",
        f"; AcVid: {folder_name}",
        f"; StreamName: {video_data['title']}",
        f"Title: {folder_name} - {video_data['user']['name']} - {video_data['title']}",
        f"Original Script: {folder_name} - {video_data['user']['name']} - {video_data['title']}",
        "Script Updated By: acfunSDK转换",
        "ScriptType: v4.00+",
        "Collisions: Normal",
        f"PlayResX: {thisVideoWidth}",
        f"PlayResY: {thisVideoHeight}"
    ])
    styles = "\n".join([
        "[V4+ Styles]",
        "Format: " + ", ".join([
            'Name', 'Fontname', 'Fontsize', 'PrimaryColour', 'SecondaryColour', 'OutlineColour',
            'BackColour', 'Bold', 'Italic', 'Underline', 'StrikeOut', 'ScaleX', 'ScaleY',
            'Spacing', 'Angle', 'BorderStyle', 'Outline', 'Shadow', 'Alignment', 'MarginL',
            'MarginR', 'MarginV', 'Encoding']),
        "Style: " + ",".join([
            'Danmu', 'Microsoft YaHei', f'{fontsize}',
            '&H00FFFFFF', '&H00FFFFFF', '&H00000000', '&H00000000',
            '0', '0', '0', '0', '100', '100', '0', '0', '1', '1',
            '0', '2', '20', '20', '2', '0'])
    ])
    events = "\n".join([
        "[Events]",
        "Format: " + ", ".join([
            'Layer', 'Start', 'End', 'Style', 'Name',
            'MarginL', 'MarginR', 'MarginV', 'Effect', 'Text\n'])
    ])
    assData = list()
    screenChannel = [None for i in range(channelNum)]

    def timeProc(second, offset=0):
        second = second + offset
        minute = math.floor(second / 60)
        hours = math.floor(second / 60 / 60)
        minute = minute - hours * 60
        second = second - hours * 60 * 60 - minute * 60
        sec = second + offset
        return f"{hours:0>2}:{minute:0>2}:{sec:0>2.2f}"

    def choice_channel(startT, endT):
        # 按新时间移除频道占位
        empty = []
        for i, thisEnd in enumerate(screenChannel):
            if i in [0, 1]:
                continue
            elif thisEnd is None:
                empty.append(i)
            elif startT > thisEnd:
                screenChannel[i] = None
        # 无空位时返回空
        if len(empty) == 0:
            return None
        # 随机选择空位，记录结束时间，返回结果
        used = random.choice(empty)
        screenChannel[used] = endT
        return used

    for danmaku in danmaku_data:
        # 略过高级弹幕
        if danmaku['danmakuType'] != 0:
            continue
        # 弹幕挂载时间（文本）（弹幕左边界 接触到 视频的右边界）
        startTime = danmaku['position'] / 1000
        # 弹幕的长度
        danmakuLen = len(danmaku['body']) * fontsize
        danmakuLen_total = danmakuLen + thisVideoWidth
        # 运动到出界的时间点
        toLeftTime = startTime + thisDuration + (danmakuLen_total / thisVideoWidth)
        # 寻找频道
        danmaku_channel = choice_channel(startTime, toLeftTime)
        if danmaku_channel is None:  # 频道全满，加速通过
            toLeftTime -= int(thisDuration / 2)
            danmaku_channel = random.randint(2, channelNum)
        channelHeight = danmaku_channel * fontsize
        # 所有点位
        x1 = danmakuLen_total
        y1 = channelHeight
        x2 = - danmakuLen
        y2 = channelHeight
        dialogue = [
            "Dialogue: 0", timeProc(startTime), timeProc(toLeftTime),
            "Danmu", f"{danmaku['userId']}", "20", "20", "2", "",
            "{\\move(" + f"{x1}", f"{y1}", f"{x2}", f"{y2})" + "}" + f"{danmaku['body']}"
        ]
        assData.append(",".join(dialogue))
    events += "\n".join(assData)
    result = "\n\n".join([scriptInfo, styles, events])
    ass_path = os.path.join(folder_path, f"{filenameId}.ass")
    with open(ass_path, 'w', encoding="utf_8_sig") as ass_file:
        ass_file.write(result)
    ass_js_path = os.path.join(folder_path, 'data', f"{filenameId}.ass.js")
    ass_js_data = [
        "let assData=\"\" + \n",
        *[f"\"{x}\" + \n" for x in result.split('\n')],
        "\"\";"
    ]
    with open(ass_js_path, 'wb') as ass_js:
        ass_js.write("".join(ass_js_data).encode())
    return ass_path


def image_uploader(client, image_data: bytes, ext: str = 'jpeg'):
    token_req = client.post(apis['image_upload_gettoken'], data=dict(fileName=uuid4().hex.upper() + f'.{ext}'))
    token_data = token_req.json()
    assert token_data.get('result') == 0
    resume_req = client.get(apis['image_upload_resume'], params=dict(upload_token=token_data['info']['token']))
    resume_data = resume_req.json()
    assert resume_data.get('result') == 1
    fragment_req = client.post(apis['image_upload_fragment'], data=image_data,
                               params=dict(upload_token=token_data['info']['token'], fragment_id=0),
                               headers={"Content-Type": "application/octet-stream"})
    fragment_data = fragment_req.json()
    assert fragment_data.get('result') == 1
    complete_req = client.post(apis['image_upload_complete'],
                               params=dict(upload_token=token_data['info']['token'], fragment_count=1))
    complete_data = complete_req.json()
    assert complete_data.get('result') == 1
    result_req = client.post(apis['image_upload_geturl'], data=dict(token=token_data['info']['token']))
    result_data = result_req.json()
    assert result_data.get('result') == 0
    return result_data.get('url')


def downloader(client, src_url, fname: [str, None] = None, dest_dir: [str, None] = None, display: bool = True):
    if dest_dir is None:
        dest_dir = os.getcwd()
    elif os.path.isabs(dest_dir) is False:
        dest_dir = os.path.abspath(dest_dir)
    if not os.path.isdir(dest_dir):
        os.makedirs(dest_dir, exist_ok=True)
    if fname is None:
        fname = urlparse(src_url).path.split('/')[-1]
    fpath = os.path.join(dest_dir, fname)

    try:
        with open(fpath, 'wb') as download_file:
            with client.stream("GET", src_url) as response:
                if response.status_code // 100 != 2:
                    download_file.close()
                    os.remove(fpath)
                    return None
                total = int(response.headers.get("Content-Length", 0))
                total = None if total == 0 else total // 1024
                downloaded = 0
                with alive_bar(total, manual=False if total is None else True,
                               length=30, disable=not display,
                               title=fname, title_length=20, force_tty=True,
                               monitor=None if total is None else "{count}/{total} [{percent:.1%}]",
                               stats=False, elapsed_end=False) as progress:
                    for chunk in response.iter_bytes():
                        download_file.write(chunk)
                        if total is None:
                            progress(int(response.num_bytes_downloaded // 1024))
                        elif total == 0:
                            progress(int((response.num_bytes_downloaded - downloaded) // 1024))
                        else:
                            progress(downloaded / total)
                        downloaded = response.num_bytes_downloaded
                    if total is not None:
                        progress(1)
    except (httpx.ConnectError, httpx.ConnectTimeout, httpx.ReadTimeout):
        print("httpx.ConnectError:", src_url)
        os.remove(fpath)
        return None
    except KeyboardInterrupt:
        os.remove(fpath)
        raise KeyboardInterrupt

    if os.path.isfile(fpath) and os.path.exists(fpath):
        if '.' not in fname:
            kind = filetype.guess_extension(fpath)
            if kind is not None:
                new_fpath = ".".join([fpath, kind])
                shutil.move(fpath, new_fpath)
                return new_fpath
        return fpath
    return None


def thin_string(_string: str, no_break: bool = False):
    final_str = list()
    for line in _string.replace('\r', '').split('\n'):
        new_line = ' '.join(line.split()).strip()
        if len(new_line):
            final_str.append(new_line)
    if no_break is True:
        return " ".join(final_str)
    return " ↲ ".join(final_str)


def warp_mix_chars(_string: str, lens: int = 40, border: [tuple, None] = None):
    output = list()
    tmp_string = ""
    tmp_count = 0
    for i, _char in enumerate(_string):
        char_count = 2 if '\u4e00' <= _char <= '\u9fcc' else 1
        if tmp_count + char_count > lens:
            tmp_string += ' ' * (lens - tmp_count)
            output.append(tmp_string)
            tmp_string = ""
            tmp_count = 0
        tmp_string += _char
        tmp_count += char_count
    if tmp_count != 0:
        tmp_string += ' ' * (lens - tmp_count)
        output.append(tmp_string)
    if isinstance(border, tuple) and len(border) == 2:
        output = [f"{border[0]}{x}{border[1]}" for x in output]
    return output


def get_channel_info(html):
    rex = re.compile('\{subChannelId\:(\d+),subChannelName\:\"((?:(?!").)*)\"\}')
    result = rex.findall(html)
    if result is not None:
        return {"subChannelId": result[0][0], "subChannelName": result[0][1]}
    return {}


def ms2time(ms: int):
    d = timedelta(milliseconds=ms)
    return str(d).split('.')[0]


def get_page_pagelets(page_obj):
    data = list()
    for item in page_obj.select("[id^=pagelet_]"):
        data.append(item.attrs['id'])
    return data


def url_complete(url):
    if isinstance(url, str):
        if url.startswith('//'):
            url = f"https:{url}"
        elif not url.startswith('http'):
            url = f"{routes['index']}{url}"
    return url


class AcLink:

    def __init__(self, acer, url, title, container=None):
        self.acer = acer
        self.title = title
        self.url = url_complete(url)
        self.container = container

    def loading(self):
        pass

    def __repr__(self):
        show_link = f" >> {self.url}" if self.url else ""
        return f"AcLink({self.title}{show_link})"


class AcImage:

    def __init__(self, acer, src, url=None, name=None, container=None):
        self.acer = acer
        self.src = url_complete(src)
        self.name = name
        self.url = url_complete(url)
        self.container = container

    def loading(self):
        pass

    def __repr__(self):
        show_link = f" >> {self.url}" if self.url else ""
        return f"AcImg({self.name}[{self.src}]{show_link})"

    def save_as(self, fname: [str, None] = None, dest_dir: [str, None] = None):
        return downloader(self.acer.client, self.url, fname, dest_dir)


class AcPagelet:
    pagelet_raw = None
    pagelet_id = None
    pagelet_obj = None
    pagelet_sp = (
        'pagelet_banner',
        'pagelet_top_area',
        'pagelet_monkey_recommend',
        'pagelet_live',
        'pagelet_spring_festival',
        'pagelet_list_banana',
        'pagelet_bangumi_list',
    )

    def __init__(self, acer, pagelet_data: [Tag, dict]):
        self.acer = acer
        self.pagelet_raw = pagelet_data
        if isinstance(pagelet_data, Tag):
            self.pagelet_id = pagelet_data.attrs['id']
            self.pagelet_obj = pagelet_data
            assert self.pagelet_id.startswith("pagelet_") or \
                   self.pagelet_id in ['footer']
        elif isinstance(pagelet_data, dict):
            self.pagelet_id = pagelet_data.get('id')
            self.pagelet_obj = Bs(pagelet_data.get('html', ""), "lxml")
        else:
            raise TypeError("pagelet_data allow bs4Tag or dict.")

    def _index_banner(self, obj=False):
        if self.pagelet_id != "pagelet_banner":
            return None
        data = dict()
        for rule in cssutils.parseString(self.pagelet_raw.get('styles', [])[0]):
            if isinstance(rule, cssutils.css.cssstylerule.CSSStyleRule) and \
                    rule.selectorText == ".page-top-banner .banner-pic":
                for pro in rule.style:
                    if pro.name == 'background-image':
                        data['image'] = pro.value[5:-2]
        data['url'] = self.pagelet_obj.select_one(".page-top-banner > a.banner-pic").attrs['href']
        data['title'] = self.pagelet_obj.select_one(".float-text").text
        if obj is True:
            return AcImage(self.acer, data['image'], data['url'], data['title'])
        return data

    def _index_top_area(self, obj=False):
        if self.pagelet_id != "pagelet_top_area":
            return None
        data = dict(slider=list(), items=list())
        for js_data in self.pagelet_raw.get('scripts', [])[0].split('\n'):
            if js_data.strip().startswith("window.sliderData = ["):
                data['slider'] = json.loads(js_data.strip()[20:-1])
        for video in self.pagelet_obj.select('a.recommend-video.log-item'):
            data['items'].append({
                'mediaid': video.attrs['data-mediaid'],
                'url': routes['video'] + video.attrs['data-mediaid'],
                'title': video.select_one('.video-title').text,
                'cover': video.select_one('img').attrs['src']
            })
        if obj is True:
            return dict(
                slider=[AcImage(
                    self.acer, s['image'], s['link'], s['title'],
                    self.acer.get(f"{routes['index']}{s['link']}")
                ) for s in data['slider']],
                items=[self.acer.AcVideo(v['mediaid'], dict(title=v['title'])) for v in data['items']]
            )
        return data

    def _index_monkey_recommend(self, obj=False):
        if self.pagelet_id != "pagelet_monkey_recommend":
            return None
        videos = list()
        for video in self.pagelet_obj.select(".monkey-recommend-videos > .video-list > .monkey-video"):
            videos.append({
                'mediaid': video.attrs['data-mediaid'],
                'url': routes['video'] + video.attrs['data-mediaid'],
                'title': video.select_one('.monkey-video-title').text,
                'infos': video.select_one('.monkey-video-title').attrs['title'].split('\r'),
                'cover': video.select_one('.monkey-video-cover').img.attrs['src'],
                'up': video.select_one('.monkey-up-name').attrs['title'],
                'up_url': video.select_one('.monkey-up-name').attrs['href'],
            })
        data = dict(items=videos)
        ad_img = self.pagelet_obj.select_one('.activity-box')
        data['ad'] = {
            'title': ad_img.attrs['data-title'],
            'url': ad_img.attrs['href'],
            'image': ad_img.img.attrs['src']
        }
        if obj is True:
            return dict(
                ad=AcImage(self.acer, data['ad']['image'], f"{data['ad']['url']}", data['ad']['title']),
                items=[
                    self.acer.AcVideo(v['mediaid'], dict(title=v['title'], user=dict(id=v['up_url'][3:], name=v['up'])))
                    for v in videos
                ]
            )
        return data

    def _index_live(self, obj=False):
        if self.pagelet_id != "pagelet_live":
            return None
        videos = list()
        for video in self.pagelet_obj.select(".live-module-videos > .video-list > .live-video"):
            videos.append({
                'liveid': video.attrs['data-liveid'],
                'url': routes['live'] + video.attrs['data-liveid'],
                'title': video.select_one('.live-video-title').text,
                'infos': video.select_one('.live-video-title').attrs['title'].split('\r'),
                'cover': video.select_one('.live-video-cover').img.attrs['src'],
                'up': video.select_one('.live-video-up-name').attrs['title'],
                'up_avatar': video.select_one('.live-video-avatar').img.attrs['src'],
            })
        data = dict(items=videos)
        ad_img = self.pagelet_obj.select_one('.activity-box')
        data['ad'] = {
            'title': ad_img.attrs['data-title'],
            'url': ad_img.attrs['href'],
            'image': ad_img.img.attrs['src']
        }
        if obj is True:
            return dict(
                ad=AcImage(self.acer, data['ad']['image'], f"{data['ad']['url']}", data['ad']['title']),
                items=[self.acer.AcLiveUp(v['liveid']) for v in videos]
            )
        return data

    def _index_spring_festival(self, obj=False):
        if self.pagelet_id != "pagelet_spring_festival":
            return None
        return None

    def _index_list_banana(self, obj=False):
        if self.pagelet_id != "pagelet_list_banana":
            return None
        data = dict(d1=list(), d3=list(), d7=list(), article=dict())
        for rank in [('d1', '.day-list'), ('d3', '.three-day-list'), ('d7', '.week-list')]:
            for video in self.pagelet_obj.select(f".rank-left > {rank[1]} > .banana-video"):
                data[rank[0]].append({
                    'mediaid': video.attrs['data-mediaid'],
                    'url': routes['video'] + video.attrs['data-mediaid'],
                    'title': video.select_one('.banana-video-title').text,
                    'infos': video.select_one('.banana-video-title').attrs['title'].split('\r'),
                    'cover': video.select_one('.banana-video-cover').img.attrs['src'],
                    'up': video.select_one('.banana-up-name').attrs['title'],
                    'up_url': video.select_one('.banana-up-name').attrs['href'],
                    'banana_count': video.select_one('.banana-count').text,
                })
        for article_tab in self.pagelet_obj.select('.rank-right .main-header-item'):
            tab_name = article_tab.select_one('.header-item-link').text
            tab_url = article_tab.select_one('.header-item-link').attrs['href']
            data['article'][tab_name] = dict(name=tab_name, url=tab_url, article=list())
            for index, article in enumerate(article_tab.select('.tab-main-content > li')):
                if index == 0:
                    this_article = {
                        'mediaid': article.attrs['data-mediaid'],
                        'url': routes['article'] + article.attrs['data-mediaid'],
                        'title': article.select_one('.main-content-block > a > p.block-title').text,
                        'infos': article.select_one('.main-content-block > a > p.block-title').attrs['title'].split(
                            '\r'),
                        'headimg': article.select_one('img.block-img').attrs['src'],
                        'up': article.select_one('span.block-up').a.attrs['title'],
                        'up_url': article.select_one('span.block-up').a.attrs['href'],
                        'comment_num': article.select_one('span.icon-comments').text
                    }
                else:
                    this_article = {
                        'mediaid': article.attrs['data-mediaid'],
                        'url': routes['article'] + article.attrs['data-mediaid'],
                        'title': article.a.text,
                        'infos': article.a.attrs['title'].split('\r'),
                    }
                data['article'][tab_name]['article'].append(this_article)
        if obj is True:
            obj_data = dict(d1=list(), d3=list(), d7=list(), article=dict())
            for rank in ['d1', 'd3', 'd7']:
                obj_data[rank] = [
                    self.acer.AcVideo(
                        v['mediaid'], dict(title=v['title'], user=dict(id=v['up_url'][3:], name=v['up'])))
                    for v in data[rank]
                ]
            for tab_name in data['article'].keys():
                obj_data['article'][tab_name] = [
                    self.acer.AcArticle(
                        a['mediaid'],
                        dict(title=a['title'], user=dict(id=a.get('up_url', '   ')[3:], name=a.get('up', ''))))
                    for a in data['article'][tab_name]['article']
                ]
            return obj_data
        return data

    def _index_bangumi_list(self, obj=False):
        if self.pagelet_id != "pagelet_bangumi_list":
            return None
        data = dict(schedule=list(), recommend=list(), anli=list())
        data['title'] = self.pagelet_obj.select_one('.area-header span.header-title').text
        data['icon'] = self.pagelet_obj.select_one('.area-header img.header-icon').attrs['src']
        data['url'] = routes['index'] + self.pagelet_obj.select_one('.header-right-more').attrs['href']
        if obj is True:
            data.update({
                'channel': self.acer.get(data['url']),
                'icon': AcImage(self.acer, data['icon'], data['url'], f"{data['title']}_icon")
            })
        for i, day in enumerate(self.pagelet_obj.select('.area-left .column-list .time-block')):
            day_list = list()
            for bangumi in day.select('.time-block .list-item'):
                if 'has-img' in bangumi.attrs['class']:
                    media_data = {
                        'mediaid': bangumi.attrs['data-mediaid'],
                        'albumid': bangumi.attrs['data-albumid'],
                        'url': routes['bangumi'] + bangumi.attrs['data-albumid'],
                        'cover': bangumi.a.img.attrs['src'],
                        'name': bangumi.select_one('a:nth-child(2) > b').text,
                        'recently': bangumi.p.text
                    }
                else:
                    media_data = {
                        'mediaid': bangumi.attrs['data-mediaid'],
                        'albumid': bangumi.attrs['data-albumid'],
                        'url': routes['bangumi'] + bangumi.attrs['data-albumid'],
                        'name': bangumi.a.b.text,
                        'recently': bangumi.a.p.text
                    }
                if obj is True:
                    day_list.append(self.acer.AcBangumi(media_data['mediaid']))
                else:
                    day_list.append(media_data)
            data['schedule'].append(day_list)
        for goood in self.pagelet_obj.select('.area-left .block-list .block-list-item'):
            media_data = {
                'mediaid': goood.attrs['data-mediaid'],
                'albumid': goood.attrs['data-albumid'],
                'url': routes['bangumi'] + goood.attrs['data-albumid'],
                'cover': goood.select_one('.block-img img').attrs['src'],
                'name': goood.select_one('.block-list-title > b > a').text,
                'follow': goood.select_one('.block-list-title > p > i.fr').text
            }
            if obj is True:
                data['recommend'].append(self.acer.AcBangumi(media_data['mediaid']))
            else:
                data['recommend'].append(media_data)
        for block in self.pagelet_obj.select('.area-right .season-rec'):
            media_data = {
                'mediaid': block.attrs['data-mediaid'],
                'albumid': block.attrs['data-albumid'],
                'url': routes['bangumi'] + block.attrs['data-albumid'],
                'cover': block.img.attrs['src']
            }
            if obj is True:
                data['anli'].append(self.acer.AcBangumi(media_data['mediaid']))
            else:
                data['anli'].append(media_data)
        return data

    def _index_pagelet_left_info(self, obj=False):
        data = dict(title=None, icon=None, links=list(), url=None)
        data['title'] = self.pagelet_obj.select_one('.module-left-header span.header-title').text
        data['icon'] = self.pagelet_obj.select_one('.module-left-header img.header-icon').attrs['src']
        for link in self.pagelet_obj.select('.link-container a'):
            href = ("" if link.attrs['href'].startswith('http') else routes['index']) + link.attrs['href']
            data['links'].append({'url': href, 'title': link.text})
        data['url'] = routes['index'] + self.pagelet_obj.select_one('.header-right-more').attrs['href']
        if obj is True:
            return {
                'channel': self.acer.get(data['url']),
                'links': [self.acer.get(x['url']) for x in data['links']],
                'icon': AcImage(self.acer, data['icon'], data['url'], f"{data['title']}_icon")
            }
        return data

    def _index_pagelet_right_rank(self, obj=False):
        data = dict(rank=dict(d1=list(), d3=list(), d7=list()))
        for index, rank in enumerate(self.pagelet_obj.select('.list-content-videos')):
            for rank_item in rank.select('.log-item'):
                rank_type = f'd{"137"[index]}'
                if 'video-item-big' in rank_item['class']:
                    rank_data = {
                        'mediaid': rank_item.attrs['data-mediaid'],
                        'url': routes['video'] + rank_item.attrs['data-mediaid'],
                        'title': rank_item.select_one('.video-title').text,
                        'infos': rank_item.select_one('.video-title').attrs['title'].split('\r'),
                        'cover': rank_item.select_one('.block-left > a > img').attrs['src'],
                        'up': rank_item.select_one('.video-up').attrs['title'],
                        'up_url': rank_item.select_one('.video-up').attrs['href'],
                        'played_num': rank_item.select_one('.video-info > .icon-view-player').text,
                        'comment_num': rank_item.select_one('.video-info > .icon-comments').text
                    }
                else:
                    rank_data = {
                        'mediaid': rank_item.attrs['data-mediaid'],
                        'url': routes['video'] + rank_item.attrs['data-mediaid'],
                        'title': rank_item.a.text,
                        'infos': rank_item.a.attrs['title'].split('\r'),
                    }
                data['rank'][rank_type].append(rank_data)
        data['rank']['url'] = self.pagelet_obj.select_one('.ranked-list > .more').attrs['href']
        if obj is True:
            obj_data = dict(d1=list(), d3=list(), d7=list())
            for k in obj_data.keys():
                obj_data[k] = [
                    self.acer.AcVideo(v['mediaid'], dict(title=v['title']))
                    for v in data['rank'][k]
                ]
            return dict(rank=obj_data)
        return data

    def _index_pagelet_big(self, obj=False):
        data = dict(items=list())
        data['url'] = routes['index'] + self.pagelet_obj.select_one('.header-right-more').attrs['href']
        for video in self.pagelet_obj.select(".module-left > div:nth-child(2) > div"):
            if 'big-image' in video['class']:
                v_data = {
                    'mediaid': video.a.attrs['href'][5:],
                    'url': routes['index'] + video.a.attrs['href'],
                    'title': video.select_one('.title').text,
                    'infos': video.select_one('.title').attrs['title'].split('\r'),
                    'cover': video.select_one('.cover > img').attrs['src'],
                    'video_time': video.select_one('.video-time').text,
                }
            else:
                this_video = video.select_one('.normal-video')
                v_data = {
                    'mediaid': this_video.attrs['data-mediaid'],
                    'url': routes['video'] + this_video.attrs['data-mediaid'],
                    'title': this_video.select_one('.normal-video-title').text,
                    'infos': this_video.select_one('.normal-video-title').attrs['title'].split('\r'),
                    'cover': this_video.select_one('.normal-video-cover').img.attrs['src'],
                    'video_time': this_video.select_one('.video-time').text,
                    'played_num': this_video.select_one('.normal-video-info > .icon-view-player').text,
                    'danmu_num': this_video.select_one('.normal-video-info > .icon-danmu').text
                }
            if obj is True:
                data['items'].append(self.acer.AcVideo(v_data['mediaid'], dict(title=v_data['title'])))
            else:
                data['items'].append(v_data)
        data.update(self._index_pagelet_left_info(obj))
        data.update(self._index_pagelet_right_rank(obj))
        return data

    def _index_pagelet_normal(self, obj=False):
        data = dict(items=list())
        for video in self.pagelet_obj.select(".normal-video-container > .normal-video"):
            v_data = {
                'mediaid': video.attrs['data-mediaid'],
                'url': routes['video'] + video.attrs['data-mediaid'],
                'title': video.select_one('.normal-video-title').text,
                'infos': video.select_one('.normal-video-title').attrs['title'].split('\r'),
                'cover': video.select_one('.normal-video-cover').img.attrs['src'],
                'video_time': video.select_one('.video-time').text,
                'played_num': video.select_one('.normal-video-info > .icon-view-player').text,
                'danmu_num': video.select_one('.normal-video-info > .icon-danmu').text
            }
            if obj is True:
                data['items'].append(self.acer.AcVideo(v_data['mediaid'], dict(title=v_data['title'])))
            else:
                data['items'].append(v_data)
        data.update(self._index_pagelet_left_info(obj))
        data.update(self._index_pagelet_right_rank(obj))
        return data

    def _footer(self, obj=False):
        if self.pagelet_id != "footer":
            return None
        data = dict(links=dict(), infos=list(), copyright=dict())
        for link_group in self.pagelet_obj.select('.footer-nav > div'):
            for i, tag in enumerate(link_group.select('h5')):
                those_links = list()
                for link in link_group.select(f'p:nth-of-type({i + 1}) a'):
                    if 'href' in link.attrs:
                        this_link = {
                            'title': link.text,
                            'url': link.attrs['href']
                        }
                        if obj is True:
                            those_links.append(AcLink(self.acer, **this_link))
                        else:
                            those_links.append(this_link)
                    else:
                        this_link = {
                            'name': link.get_text(),
                            'src': link.select_one('img').attrs['src']
                        }
                        if obj is True:
                            those_links.append(AcImage(self.acer, **this_link))
                        else:
                            those_links.append(this_link)
                data['links'][tag.text] = those_links
        for info in self.pagelet_obj.select('.footer-link a'):
            this_link = {
                'title': info.text.strip(),
                'url': info.attrs['href']
            }
            if obj is True:
                data['infos'].append(AcLink(self.acer, **this_link))
            else:
                data['infos'].append(this_link)
        for ac in self.pagelet_obj.select_one('.footer-link > div:last-child').select('span'):
            if ": " in ac.text:
                k, v = ac.text.split(": ")
                data['copyright'][k] = v
        return data

    def to_dict(self, obj=False):
        if self.pagelet_id == 'footer':
            return self._footer(obj)
        elif self.pagelet_id in self.pagelet_sp:
            func_name = self.pagelet_id.replace("pagelet_", "_index_")
            return getattr(self, func_name)(obj)
        elif self.pagelet_id in pagelets_big:
            return self._index_pagelet_big(obj)
        elif self.pagelet_id in pagelets_normal:
            return self._index_pagelet_normal(obj)
        return None


class AcComment:
    sourceId = None
    hot_comments = list()
    root_comments = list()
    sub_comments = dict()
    commentIds = list()
    commentsMap = dict()

    def __init__(self, acer, sid: [str, int], stype: int = 3, referer: [str, None] = None):
        self.acer = acer
        self.sourceId = str(sid)
        self.sourceType = stype
        self.referer = referer or f"{routes['index']}"

    def __repr__(self):
        return f"AcComment([ac{self.sourceId}] Σ{len(self.root_comments)})"

    def _get_data(self, page: int = 1, api_name: str = 'comment'):
        assert api_name in ['comment', 'comment_floor']
        param = {
            "sourceId": self.sourceId,
            "sourceType": self.sourceType,
            "page": page,
            "pivotCommentId": 0,
            "newPivotCommentId": "",
            "t": str(time.time_ns())[:13],
            "supportZtEmot": True,
        }
        req = self.acer.client.get(apis[api_name], params=param)
        return req.json()

    def _get_sub(self, root_id, page: int = 1):
        param = {
            "sourceId": self.sourceId,
            "sourceType": self.sourceType,
            "rootCommentId": root_id,
            "page": page,
            "t": str(time.time_ns())[:13],
            "supportZtEmot": True
        }
        req = self.acer.client.get(apis['comment_subs'], params=param)
        return req.json()

    def get_all_comment(self):
        self.hot_comments = list()
        self.root_comments = list()
        self.sub_comments = dict()
        page = 1
        page_max = 10
        with alive_bar(100, manual=True, length=30, title="get all comments", force_tty=True, stats=False) as bar:
            while page <= page_max:
                api_data = self._get_data(page)
                if api_data.get('result') != 0:
                    print(api_data)
                    break
                self.hot_comments.extend(api_data.get('hotComments', []))
                self.root_comments.extend(api_data.get('rootComments', []))
                self.sub_comments.update(api_data.get('subCommentsMap', {}))
                page_max = api_data.get('totalPage', page)
                page = api_data.get('curPage', 1)
                page += 1
                bar(page / page_max)
            bar(1)

        for rid, sub_data in self.sub_comments.items():
            while sub_data['pcursor'] != "no_more":
                sub_page = self._get_sub(rid, sub_data['pcursor'])
                if 'subComments' not in sub_page:
                    sub_data['pcursor'] = "no_more"
                    break
                sub_data['subComments'].extend(sub_page['subComments'])
                if sub_page['curPage'] < sub_page['totalPage']:
                    sub_data['pcursor'] += 1
                else:
                    sub_data['pcursor'] = "no_more"
                time.sleep(0.1)

    def get_all_floors(self):
        first_page = self._get_data(1, 'comment_floor')
        self.commentIds = first_page['commentIds']
        self.commentsMap = first_page['commentsMap']
        page = first_page['curPage']
        page_max = first_page['totalPage']
        while page <= page_max:
            page += 1
            api_data = self._get_data(page, 'comment_floor')
            assert api_data['result'] == 0
            self.commentIds.extend(api_data['commentIds'])
            self.commentsMap.update(api_data['commentsMap'])
            time.sleep(0.5)

    @need_login
    def add(self, content: str, reply_id: [str, int, None] = None):
        form_data = {
            "sourceId": self.sourceId,
            "sourceType": self.sourceType,
            "content": content,
            "replyToCommentId": reply_id or "",
        }
        req = self.acer.client.post(apis['comment_add'], data=form_data,
                                    headers={'referer': self.referer})
        return req.json().get('result') == 0

    @need_login
    def delete(self, floor: int):
        this_comment = None
        for c in self.root_comments:
            if c.get('floor') == floor:
                this_comment = Comment(self.acer, c, self)
                break
        if this_comment is None:
            return False
        return this_comment.delete()

    def hot(self):
        datas = list()
        for c in self.hot_comments:
            datas.append(Comment(self.acer, c, self))
        return datas

    def list(self):
        datas = list()
        for c in self.root_comments:
            datas.append(Comment(self.acer, c, self))
        return datas

    def get(self, floor: int):
        for x in self.root_comments:
            if x['floor'] == floor:
                return Comment(self.acer, x, self)
        return None

    def find(self, cid: [int, str]):
        for x in self.root_comments:
            if x['commentId'] == int(cid):
                return Comment(self.acer, x, self)
        for x in self.sub_comments.values():
            for y in x.get('subComments', []):
                if y['commentId'] == int(cid):
                    return Comment(self.acer, y, self)
        return None


class Comment:
    data = dict()
    ac_num = None

    def __init__(self, acer, data: dict, main_obj: AcComment):
        self.acer = acer
        self.data = data
        self.ac_num = self.data.get('sourceId')
        self.main_obj = main_obj
        self.referer = self.main_obj.referer or f"{routes['index']}"

    def cdata(self, name):
        return self.data.get(name)

    def __repr__(self):
        content = self.data.get('content', '').replace('\r\n', '↲  ').replace('\n', '↲  ')
        return f"C([{self.cdata('commentId')}]#{self.cdata('floor')} {content} " \
               f"@{self.cdata('userName')}[{self.cdata('userId')}])"

    @property
    def api_data(self):
        return {
            "sourceId": self.data.get('sourceId'),
            "sourceType": self.data.get('sourceType'),
            "commentId": self.data.get('commentId'),
        }

    @need_login
    def like(self):
        if self.cdata('isLiked') is False:
            req = self.acer.client.post(apis['comment_like'], data=self.api_data, headers={'referer': self.referer})
            return req.json().get('result') == 0
        return True

    @need_login
    def unlike(self):
        if self.cdata('isLiked') is True:
            req = self.acer.client.post(apis['comment_unlike'], data=self.api_data, headers={'referer': self.referer})
            return req.json().get('result') == 0
        return True

    @need_login
    def delete(self):
        req = self.acer.client.post(apis['comment_delete'], data=self.api_data, headers={'referer': self.referer})
        return req.json().get('result') == 0


class AcDanmaku:
    ac_num = None
    vid = None
    danmaku_data = list()
    video_data = dict()

    def __init__(self, acer, video_data: dict):
        self.video_data = video_data
        self.ac_num = self.video_data.get('dougaId', self.video_data.get('bangumiId'))
        self.vid = self.video_data.get('currentVideoId', self.video_data.get('videoId'))
        self.acer = acer
        self._get_all_danmaku()

    def __repr__(self):
        return f"AcDanmaku([{self.vid}] Σ{len(self.danmaku_data)})"

    def _get_data(self, page=1, limit=200, sorttype=2, asc=True):
        param = {
            "resourceId": f"{self.vid}",
            "resourceType": "9",
            "enableAdvanced": True,
            "pcursor": f"{page}",
            "count": f"{limit}",
            "sortType": f"{sorttype}",
            "asc": asc,
        }
        req = self.acer.client.get(apis['danmaku'], params=param)
        return req.json()

    def _get_all_danmaku(self):
        self.danmaku_data = list()
        limit = 200
        page = 1
        more = 1
        while more > 0:
            api_data = self._get_data(page)
            total = api_data.get('totalCount', 1)
            self.danmaku_data.extend(api_data.get('danmakus', []))
            more = math.ceil(total / limit) - page
            page += 1

    def list(self):
        return [Danmaku(x, self.acer, self.video_data) for x in self.danmaku_data]

    @need_login
    def add(self, words, ms, color=16777215, mode=1, size=25):
        danmaku = {
            "size": size,  # 大* 中25 小16
            "mode": mode,
            "color": color,
            "position": ms,
            "body": words,
            "type": "bangumi" if "bangumiTitle" in self.video_data else "douga",
            "videoId": self.vid,
            "id": self.ac_num,
            "subChannelId": self.video_data.get('subChannelId'),
            "subChannelName": self.video_data.get('subChannelName'),
            "roleId": ""
        }
        req = self.acer.client.post(apis['danmaku_add'], data=danmaku, headers={"referer": f"{routes['index']}"})
        return req.json().get('result') == 0


class Danmaku:
    data = dict()
    video_data = dict()
    ac_num = None
    vid = None
    isLike = None

    def __init__(self, data: dict, acer=None, video_data: [dict, None] = None):
        self.data = data
        self.acer = acer
        if isinstance(video_data, dict):
            self.video_data = video_data
            self.vid = self.video_data.get('currentVideoId')
            self.ac_num = self.video_data.get('dougaId')

    def __repr__(self):
        return f"DM([{ms2time(self.position)}]#{self.danmakuId} {self.body} @{self.userId})"

    def __getattr__(self, item):
        if item in self.data.keys():
            return self.data.get(item)
        return super().__getattribute__(item)

    def get_up(self):
        return self.acer.ac_up({'id': self.userId}, self.acer)

    @need_login
    def like(self):
        req = self.acer.client.post(apis['danmaku_like'], params={"danmakuId": self.danmakuId})
        self.isLike = req.json().get('result') == 0
        return self.isLike is True

    @need_login
    def like_cancel(self):
        req = self.acer.client.post(apis['danmaku_like_cancel'], params={"danmakuId": self.danmakuId})
        self.isLike = not (req.json().get('result') == 0)
        return self.isLike is False

    @need_login
    def block_words(self, word=None):
        req = self.acer.client.post(apis['danmaku_block_add'], params={
            "blockWordsType": 1,
            "blockWords": word or self.body
        })
        return req.json().get('result') == 0

    @need_login
    def block_acer(self):
        req = self.acer.client.post(apis['danmaku_block_add'], params={
            "blockWordsType": 2,
            "blockWords": self.userId
        })
        return req.json().get('result') == 0

    @need_login
    def block_words_delete(self):
        req = self.acer.client.post(apis['danmaku_block_delete'], params={
            "blockWordsType": 1,
            "blockWordsList": self.body
        })
        return req.json().get('result') == 0

    @need_login
    def block_acer_delete(self):
        req = self.acer.client.post(apis['danmaku_block_delete'], params={
            "blockWordsType": 2,
            "blockWordsList": self.userId
        })
        return req.json().get('result') == 0

    @need_login
    def report(self):
        req = self.acer.client.post(apis['danmaku_block_delete'], params={
            "reportedUserId": self.userId,
            "danmakuId": self.danmakuId,
            "body": self.body,
            "type": "douga",
            "id": self.ac_num,
            "videoId": self.vid,
            "subChannelId": self.video_data.get('channel', {}).get('id'),
            "subChannelName": self.video_data.get('channel', {}).get('name'),
        })
        return req.json().get('result') == 0


class Message:
    intro = None
    create_at = None
    whom = None
    raw_data = None
    raw_link = None

    def __init__(self, acer, **kwargs):
        self.acer = acer
        self.raw_data = kwargs
        self.raw_link = kwargs.get('content_url')
        self.intro = kwargs.get('intro')
        self.create_at = kwargs.get('create_at')
        if 'uid' in kwargs:
            self.whom = {'userId': kwargs.get('uid'), 'name': kwargs.get('username')}

    def __repr__(self):
        return f"AcMsg({self.intro})"

    def user(self):
        if self.whom is None:
            return None
        return self.acer.AcUp(self.whom)


class ReplyMsg(Message):

    def __init__(self, acer, **kwargs):
        self.content_title = kwargs.get('content_title', '')
        self.replied = kwargs.get('replied', '')
        self.content = kwargs.get('content', '')
        self.ncid = kwargs.get('ncid', '')
        self.username = kwargs.get('username', '')
        super().__init__(acer, **kwargs)

    def __repr__(self):
        return f"AcReply({self.content_title}—— {self.content} @{self.username})"

    def content(self):
        return self.acer.get(self.raw_data.get('content_url'))

    def replay(self):
        comments = self.content().comment()
        return comments.find(self.raw_data.get('ncid'))


class LikeMsg(ReplyMsg):

    def __init__(self, acer, **kwargs):
        super().__init__(acer, **kwargs)

    def __repr__(self):
        return f"AcLike({self.replied} | @{self.username})"


class AtMsg(ReplyMsg):

    def __init__(self, acer, **kwargs):
        super().__init__(acer, **kwargs)

    def __repr__(self):
        return f"AcMsg({self.raw_link}#ncid={self.ncid} | @{self.username})"


class GiftMsg(Message):
    banana = 0

    def __init__(self, acer, **kwargs):
        self.content_title = kwargs.get('content_title', '')
        self.username = kwargs.get('username', '')
        self.banana = kwargs.get('banana', 0)
        super().__init__(acer, **kwargs)

    def __repr__(self):
        return f"AcGift({self.content_title} | 🍌x{self.banana} @{self.username})"


class NoticeMsg(Message):

    def __init__(self, acer, **kwargs):
        self.content_title = kwargs.get('content_title', '')
        super().__init__(acer, **kwargs)

    def __repr__(self):
        return f"AcNotice({self.content_title} >>> {self.raw_link})"


class SystemMsg(Message):
    classify = None

    def __init__(self, acer, **kwargs):
        super().__init__(acer, **kwargs)

    def __repr__(self):
        if 'up' in self.raw_data and '关注了你' in self.raw_data.get('intro', ''):
            link = self.raw_data.get('up', [])
            return f"AcFans(+1 | @{link[0]})"
        elif 'video' in self.raw_data and '已通过审核' in self.raw_data.get('intro', ''):
            link = self.raw_data.get('video', [])
            return f"AcPass({link[0]} 已通过审核)"
        elif 'article' in self.raw_data and '已通过审核' in self.raw_data.get('intro', ''):
            link = self.raw_data.get('article', [])
            return f"AcPass({link[0]} 已通过审核)"
        elif '有人收藏了你的' in self.raw_data.get('intro', ''):
            if 'video' in self.raw_data:
                link = self.raw_data.get('video', [])
            elif 'article' in self.raw_data:
                link = self.raw_data.get('article', [])
            else:
                return f"AcMsg({self.intro})"
            return f"AcStar(+1 | {link[0]})"
        return f"AcMsg({self.intro})"


class AcMessage:
    req_count = 0

    def __init__(self, acer):
        self.acer = acer

    @property
    def unread(self):
        if self.acer.is_logined is False:
            return None
        api_req = self.acer.client.get(apis['unread'])
        api_data = api_req.json()
        if api_data.get('result') != 0:
            return None
        return api_data.get("unReadCount")

    def _get_msg_api(self, vid: str = '', page: int = 1):
        vids = ['', 'like', 'atmine', 'gift', 'sysmsg', 'resmsg']
        assert vid in vids
        self.req_count += 1
        param = {
            "quickViewId": 'upCollageMain',
            "reqID": self.req_count,
            "ajaxpipe": 1,
            "pageNum": page,
            "t": str(time.time_ns())[:13],
        }
        api_req = self.acer.client.get(apis['message'] + vid, params=param)
        if api_req.text.endswith("/*<!-- fetch-stream -->*/"):
            api_data = json.loads(api_req.text[:-25])
            page_obj = Bs(api_data.get('html', ''), 'lxml')
        else:
            return None
        item_data = list()
        total = str(page_obj.select_one('#listview').attrs['totalcount'])
        for item in page_obj.select('#listview > ul,.main-block-msg-item'):
            if vid == '':
                main_url = scheme + ':' + item.select_one('.intro').a.attrs['href']
                reply_url = item.select_one('a.msg-reply').attrs['href']
                item_data.append({
                    'content_url': main_url,
                    'content_title': item.select_one('.intro').a.text.strip(),
                    'replied': item.select_one('.msg-replied .inner').text.strip().replace('\xa0', ' '),
                    'uid': item.select_one('.titlebar .name').attrs['href'][17:],
                    'username': item.select_one('.titlebar .name').text,
                    'create_at': item.select_one('.titlebar .time').text.strip(),
                    'ncid': reply_url.split('#ncid=')[1],
                    'content': item.select_one('.msg-reply .inner').text.strip().replace('\xa0', ' '),
                    'intro': item.select_one('.content .intro').text.strip(),
                })
            elif vid == 'like':
                this_url = item.select_one('a.replied').attrs['href'].split('#')
                main_url = scheme + ':' + this_url[0]
                item_data.append({
                    'content_url': main_url,
                    'replied': item.select_one('.clamp-text .inner').text.strip().replace('\xa0', ' '),
                    'uid': item.select_one('.titlebar .name').attrs['href'][17:],
                    'username': item.select_one('.titlebar .name').text,
                    'ncid': this_url[1][5:],
                    'create_at': item.select_one('.titlebar span.time').text.strip(),
                    'intro': item.select_one('.titlebar').text.strip(),
                })
            elif vid == 'atmine':
                this_url = item.select_one('.content .msg-text').attrs['href']
                item_data.append({
                    'content_url': scheme + ':' + this_url.split('#')[0],
                    'ncid': this_url.split('#ncid=')[1],
                    'uid': item.select_one('.avatar-section').attrs['href'][17:],
                    'username': item.select_one('.titlebar-container .name').text,
                    'create_at': item.select_one('.titlebar-container span.time').text.strip(),
                    'intro': item.select_one('.titlebar-container .intro').text.strip(),
                })
            elif vid == 'gift':
                if 'moment-gift' in item.attrs['class']:
                    this_url = item.select_one('.msg-content').a.attrs['href']
                    intro = thin_string(item.select_one('.msg-content').text)
                    item_data.append({
                        'classify': 'moment',
                        'content_url': scheme + ':' + this_url,
                        'content_title': '动态',
                        'uid': item.select_one('.avatar-section').attrs['href'][17:],
                        'username': item.select_one('.content .name').text,
                        'create_at': item.select_one('.content span.time').text.strip(),
                        'intro': intro,
                        'banana': int(re.findall('(\d)根香蕉', intro)[0])
                    })
                else:
                    acer_url = item.select_one('p a:nth-of-type(1)')
                    this_url = item.select_one('p a:nth-of-type(2)')
                    intro = thin_string(item.select_one('p').text)
                    item_data.append({
                        'classify': 'content',
                        'content_url': scheme + ':' + this_url.attrs['href'],
                        'content_title': this_url.text,
                        'uid': acer_url.attrs['href'][17:],
                        'username': acer_url.text,
                        'create_at': item.select_one('.msg-item-time').text.strip(),
                        'intro': intro,
                        'banana': int(re.findall('(\d)根香蕉', intro)[0])
                    })
            elif vid == 'sysmsg':
                this_title = item.select_one('div:nth-of-type(1)').text.strip()
                this_content = item.select_one('div:nth-of-type(2)').get_text("|", strip=True)
                this_content = "\n".join([text for text in this_content.split("|") if not text.startswith('http')])
                this_content = re.sub('(>{2,})', '', this_content)
                this_link = item.select_one('div:nth-of-type(2)').a.attrs['href']
                item_data.append({
                    'content_url': this_link,
                    'content_title': this_title,
                    'create_at': item.select_one('.msg-item-time').text.strip(),
                    'intro': thin_string(this_content),
                })
            elif vid == 'resmsg':
                intro = item.select_one('p:nth-of-type(1)').text.strip()
                links = dict()
                for link in item.select('a'):
                    url_str = link.attrs['href']
                    if not url_str.startswith('http'):
                        url_str = scheme + ':' + url_str
                    for link_name in ['video', 'article', 'album', 'bangumi', 'up', 'live']:
                        if url_str.startswith(routes[link_name]) and link_name not in links:
                            links[link_name] = [link.text, url_str]
                item_data.append({
                    'create_at': item.select_one('p.msg-item-time').text.strip(),
                    'intro': thin_string(intro, True),
                    **links
                })
        return item_data, total

    def reply(self, page: int = 1, obj: bool = False):
        api_data, total = self._get_msg_api('', page)
        if obj is True:
            return [ReplyMsg(self, **i) for i in api_data]
        return api_data

    def like(self, page: int = 1, obj: bool = False):
        api_data, total = self._get_msg_api('like', page)
        if obj is True:
            return [LikeMsg(self, **i) for i in api_data]
        return api_data

    def at(self, page: int = 1, obj: bool = False):
        api_data, total = self._get_msg_api('atmine', page)
        if obj is True:
            return [AtMsg(self, **i) for i in api_data]
        return api_data

    def gift(self, page: int = 1, obj: bool = False):
        api_data, total = self._get_msg_api('gift', page)
        if obj is True:
            return [GiftMsg(self, **i) for i in api_data]
        return api_data

    def notice(self, page: int = 1, obj: bool = False):
        api_data, total = self._get_msg_api('sysmsg', page)
        if obj is True:
            return [NoticeMsg(self, **i) for i in api_data]
        return api_data

    def system(self, page: int = 1, obj: bool = False):
        api_data, total = self._get_msg_api('resmsg', page)
        if obj is True:
            return [SystemMsg(self, **i) for i in api_data]
        return api_data



#!/usr/bin/env python

from .acfun import *
from .alive import *
# from .archive import *
# from .baidu import *
# from .bandcamp import *
# from .bigthink import *
from .bilibili import *
from .bokecc import *
# from .cbs import *
# from .ckplayer import *
from .cntv import *
# from .coub import *
from .dailymotion import *
# from .douban import *
from .douyin import *
from .douyutv import *
# from .ehow import *
# from .facebook import *
# from .fc2video import *
# from .flickr import *
# from .freesound import *
# from .funshion import *
# from .google import *
# from .heavymusic import *
# from .icourses import *
# from .ifeng import *
# from .imgur import *
# from .infoq import *
# from .instagram import *
# from .interest import *
# from .iqilu import *
from .iqiyi import *
# from .joy import *
# from .khan import *
# from .ku6 import *
# from .kakao import *
# from .kuaishou import *
# from .kugou import *
# from .kuwo import *
from .le import *
# from .lizhi import *
# from .longzhu import *
# from .magisto import *
# from .metacafe import *
# from .mgtv import *
# from .miaopai import *
# from .miomio import *
# from .mixcloud import *
# from .mtv81 import *
# from .nanagogo import *
# from .naver import *
from .netease import *
from .nicovideo import *
# from .pinterest import *
# from .pixnet import *
# from .pptv import *
from .qie import *
# from .qingting import *
from .qq import *
# from .showroom import *
from .sina import *
# from .sohu import *
# from .soundcloud import *
# from .suntv import *
# from .ted import *
# from .theplatform import *
from .tiktok import *
# from .tucao import *
from .tudou import *
from .tumblr import *
from .twitter import *
# from .ucas import *
# from .veoh import *
from .vimeo import *
from .vine import *
# from .vk import *
# from .w56 import *
# from .wanmen import *
# from .xinpianchang import *
# from .yixia import *
from .youku import *
from .youtube import *
# from .zhanqi import *
# from .zhibo import *
# from .zhihu import *



# coding=utf-8
from .index import AcIndex, AcDownload
from .channel import AcChannel, AcWen
from .rank import AcRank
from .search import AcSearch
from .acer import AcUp
from .article import AcArticle
from .video import AcVideo
from .bangumi import AcBangumi
from .album import AcAlbum
from .moment import AcMoment
from .live import AcLive, AcLiveUp
from .utils import AcComment, AcLink, AcImage, AcMessage, B64s



# coding=utf-8
import os
import json
from bs4 import BeautifulSoup as Bs
from acfun.source import routes, apis
from acfun.page.utils import ms2time, get_channel_info, get_page_pagelets, AcDanmaku
from acfun.libs.you_get.extractors.acfun import download as you_get_download
from acfun.saver import VideoSaver

__author__ = 'dolacmeo'


class AcVideo:
    ac_num = None
    page_obj = None
    page_pagelets = []
    video_data = dict()
    vid = None
    resourceType = 9
    is_404 = False

    def __init__(self, acer, ac_num: [str, int], video_data: [dict, None] = None):
        if isinstance(ac_num, str) and ac_num.startswith('ac'):
            ac_num = ac_num[2:]
        self.ac_num = str(ac_num)
        self.part_num = 1
        self.acer = acer
        if isinstance(video_data, dict):
            self.video_data = video_data
        if "_" in self.ac_num:
            self.ac_num, self.part_num = self.ac_num.split('_')
        self.loading()

    @property
    def referer(self):
        if int(self.part_num) == 1:
            return f"{routes['video']}{self.ac_num}"
        return f"{routes['video']}{self.ac_num}_{self.part_num}"

    @property
    def share_url(self):
        return self.referer

    @property
    def mobile_url(self):
        return f"https://scan.acfun.cn/vd/{self.ac_num}"

    def __repr__(self):
        if self.is_404:
            return f"AcVideo([ac{self.ac_num}]咦？世界线变动了。看看其他内容吧~)"
        title = self.video_data.get('title', "")
        user_name = self.video_data.get('user', {}).get('name', "") or self.video_data.get('user', {}).get('id', "")
        user_txt = "" if len(user_name) == 0 else f" @{user_name}"
        duration = self.video_data.get('durationMillis', 0)
        duration_txt = "" if duration == 0 else f"[{ms2time(duration)}]"
        return f"AcVideo([ac{self.ac_num}]{duration_txt}{title}{user_txt})".encode(errors='replace').decode()

    def loading(self):
        req = self.acer.client.get(self.referer)
        self.is_404 = req.status_code == 404
        if self.is_404:
            return False
        self.page_obj = Bs(req.text, 'lxml')
        js_code = self.page_obj.select_one("#pagelet_newheader").find_next_sibling("script").text.strip().split('\n')[0]
        js_code = "".join(js_code.split())
        js_head = "window.pageInfo=window.videoInfo="
        js_end = ";"
        assert js_code.startswith(js_head) and js_code.endswith(js_end)
        self.video_data = json.loads(js_code[len(js_head):-len(js_end)])
        self.video_data.update(get_channel_info(req.text))
        self.vid = self.video_data.get("currentVideoId")
        self.page_pagelets = get_page_pagelets(self.page_obj)
        staff_data = self.staff()
        if staff_data is not None:
            self.video_data['staffInfos'] = staff_data.get('staffInfos')
            self.video_data['upInfo'] = staff_data.get('upInfo')

    def saver(self, dest_path=None):
        return VideoSaver(self.acer, self, dest_path)

    @property
    def video_list(self):
        return self.video_data.get('videoList', [])

    def set_video(self, num=1):
        assert num <= len(self.video_list)
        self.part_num = num
        self.loading()
        return True

    def download(self, num=1):
        self.set_video(num)
        v_num = f"_{num}" if num > 1 else ""
        video_download_path = os.path.join(self.acer.DOWNLOAD_PATH, 'video')
        you_get_download(self.referer + v_num, output_dir=video_download_path, merge=True)
        with open(os.path.join(video_download_path, f"ac{self.ac_num}.json"), 'w') as jfile:
            json.dump(self.video_data, jfile)
            jfile.close()
        return True

    def up(self):
        return self.acer.AcUp(self.video_data.get('user', {}))

    def staff(self):
        if self.video_data.get('staffContribute') is not True:
            return None
        form_data = {"resourceId": self.ac_num, "resourceType": 2}
        api_req = self.acer.client.post(apis['getStaff'], data=form_data)
        api_data = api_req.json()
        return api_data

    def danmaku(self):
        return AcDanmaku(self.acer, self.video_data)

    def comment(self):
        return self.acer.AcComment(self.ac_num, 3, self.referer)

    def like(self):
        return self.acer.like(self.ac_num, 2)

    def like_cancel(self):
        return self.acer.like_cancel(self.ac_num, 2)

    def favorite_add(self, folder_id: [str, None] = None):
        return self.acer.favourite.add(self.ac_num, self.resourceType, folder_id)

    def favorite_cancel(self, folder_id: [str, None] = None):
        return self.acer.favourite.cancel(self.ac_num, self.resourceType, folder_id)

    def banana(self, count: int):
        return self.acer.throw_banana(self.ac_num, 2, count)

    # 一键奥里给！
    def aoligei(self, danmu: bool = False, comment: bool = False):
        """ 赞 藏 蕉 弹 评 """
        """ 👍 🔖 🍌 🌠 💬 """
        print(self.like())  # 👍 点赞
        print(self.favorite_add())  # 🔖 收藏
        print(self.banana(5))  # 🍌 投蕉
        if danmu is True:  # 🌠 发弹幕
            self.danmaku().add("棒棒棒~加油哦~", 0)
        if comment is True:  # 💬 留言
            self.comment().add('<p><font color="#ff0000">棒棒棒~加油哦~</font></p>'
                               '<p><font color="#c4bd97">from  acfunSDK</font></p>')
        print(f" 赞 藏 蕉 弹 评 \n 👍 🔖 🍌 🌠 💬 \n 分享：{self.referer}?shareUid={self.acer.uid}")
        return True



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: GifAttachment.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x13GifAttachment.proto\x12\x15\x41\x63\x46unDanmu.Im.Message\"n\n\rGifAttachment\x12\x10\n\x08mimeType\x18\x01 \x01(\t\x12\x0b\n\x03url\x18\x02 \x01(\t\x12\r\n\x05width\x18\x03 \x01(\x05\x12\x0e\n\x06height\x18\x04 \x01(\x05\x12\x11\n\tthumbaUrl\x18\x05 \x01(\t\x12\x0c\n\x04text\x18\x06 \x01(\tb\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'GifAttachment_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _GIFATTACHMENT._serialized_start=46
  _GIFATTACHMENT._serialized_end=156
# @@protoc_insertion_point(module_scope)



# coding=utf-8
import json
import os
import time
from bs4 import BeautifulSoup as Bs
from acfun.source import scheme, routes, apis, pagelets_from_page, pagelets_from_api
from acfun.page.utils import downloader, get_page_pagelets, AcPagelet

__author__ = 'dolacmeo'


class AcIndex:
    index_obj = None
    index_pagelets = []
    nav_data = dict()

    def __init__(self, acer=None):
        self.acer = acer
        self._get_index()

    def _get_index(self):
        req = self.acer.client.get(routes['index'])
        self.index_obj = Bs(req.text, 'lxml')
        self.index_pagelets = get_page_pagelets(self.index_obj)

    def _get_pagelet_inner(self, area: [str, None] = None):
        datas = dict()
        for js in self.index_obj.select("script"):
            if js.text.startswith("bigPipe.onPageletArrive"):
                data = json.loads(js.text[24:-2])
                datas[data['id']] = data
        datas['footer'] = self.index_obj.select_one('#footer')
        if isinstance(area, str):
            return datas.get(area)
        return datas

    def _get_pagelet_api(self, area):
        assert area in pagelets_from_api
        param = {
            "pagelets": area, "reqID": 0, "ajaxpipe": 1,
            "t": str(time.time_ns())[:13]
        }
        req = self.acer.client.get(routes['index'] + '/', params=param)
        if req.text.endswith("/*<!-- fetch-stream -->*/"):
            return json.loads(req.text[:-25])
        return req.json()

    def nav_list(self):
        navs = list()
        for cid in self.acer.nav_data.keys():
            navs.append(self.acer.AcChannel(cid))
        return navs

    def get(self, area: str, obj=False):
        if area != 'footer' and not area.startswith("pagelet_"):
            area = "pagelet_" + area
        if area in pagelets_from_page:
            raw_data = self._get_pagelet_inner(area)
        elif area in pagelets_from_api:
            raw_data = self._get_pagelet_api(area)
        else:
            raise ValueError('area not support')
        acp = AcPagelet(self.acer, raw_data)
        return acp.to_dict(obj)


class AcDownload:
    default_dest = os.path.join(os.getcwd(), 'download')

    def __init__(self, acer):
        self.acer = acer
        if os.path.exists(self.default_dest) is False:
            os.mkdir(self.default_dest)

    def emots(self, dest_path: [str, None] = None):
        dest_path = self.default_dest if dest_path is None else dest_path
        emot_page = self.acer.client.get(routes['emot'])
        emot_obj = Bs(emot_page.text, 'lxml')
        for item in emot_obj.select('.emot-download a.download-btn'):
            furl = f"{scheme}:{item.attrs['href']}"
            downloader(self.acer.client, furl, dest_dir=dest_path)
        return dest_path

    def Android_apk(self, dest_path: [str, None] = None):
        dest_path = self.default_dest if dest_path is None else dest_path
        api_req = self.acer.client.get(apis['app_download'])
        api_data = api_req.json()
        downloader(self.acer.client, api_data.get('url'), dest_dir=dest_path)
        return dest_path

    def _app_page_obj(self):
        page_req = self.acer.client.get(routes['app'])
        return Bs(page_req.text, 'lxml')

    def LiveMate_win(self, dest_path: [str, None] = None):
        dest_path = self.default_dest if dest_path is None else dest_path
        win_link = self._app_page_obj().select_one('.zbbl-info .download a.win')
        downloader(self.acer.client, win_link.attrs['href'], dest_dir=dest_path)
        return dest_path

    def VirtualView_win(self, dest_path: [str, None] = None):
        dest_path = self.default_dest if dest_path is None else dest_path
        win_link = self._app_page_obj().select_one('.mbzs-info .download a.win')
        downloader(self.acer.client, win_link.attrs['href'], dest_dir=dest_path)
        psd_link = self._app_page_obj().select_one('.mbzs-info .dis a.download-psd')
        downloader(self.acer.client, psd_link.attrs['href'], dest_dir=dest_path)
        return dest_path

    def FaceCatcher_win(self, dest_path: [str, None] = None):
        dest_path = self.default_dest if dest_path is None else dest_path
        api_res = self.acer.client.post(apis['face_catcher'], headers={
            "referer": "https://www.acfun.cn/face-catcher",
            "origin": "https://www.acfun.cn"
        })
        api_data = api_res.json()
        downloader(self.acer.client, api_data['downloadUrl'], dest_dir=dest_path)
        downloader(self.acer.client, api_data['psdUrl'], "face_catcher_psd.zip", dest_dir=dest_path)
        return dest_path

    def emot_zips(self):
        dest_path = os.path.join(self.default_dest, 'emotionPackage')
        if not os.path.exists(dest_path):
            os.mkdir(dest_path)
        api_res = self.acer.client.post(apis['emot'], headers={
            "referer": "https://www.acfun.cn/info/",
            "origin": "https://www.acfun.cn"
        })
        api_data = api_res.json()
        for em in api_data['emotionPackageList']:
            if em['downloadUrl']:
                downloader(self.acer.client, em['downloadUrl'], f"{em['name']}.zip", dest_dir=dest_path)
        return dest_path



# coding=utf-8
import json
import re
from bs4 import BeautifulSoup as Bs
from acfun.source import routes, apis
from acfun.page.utils import thin_string
from acfun.exceptions import *

__author__ = 'dolacmeo'


class Moment:
    raw_data = dict()
    like_count = 0
    banana_count = 0
    comment_count = 0
    linked = None

    def __init__(self, acer, raw_data: dict):
        self.acer = acer
        self.raw_data = raw_data
        if self.tag_rtype in [1, 2, 3, 10]:
            if self.rtype == 3:
                self.linked = self.acer.AcArticle(self.rid)
            elif self.rtype == 2:
                self.linked = self.acer.AcVideo(self.rid)
            elif self.rtype == 10:
                rsource = self.raw_data.get('repostSource')
                if rsource is not None:
                    self.linked = Moment(self.acer, self.raw_data.get('repostSource'))

    @property
    def tag_rtype(self):
        return self.raw_data.get('tagResourceType')

    @property
    def rtype(self):
        return self.raw_data.get('resourceType')

    @property
    def rid(self):
        return self.raw_data.get('resourceId')

    @property
    def mtype(self):
        return self.raw_data.get('momentType')

    def __repr__(self):
        if self.tag_rtype in [1, 2, 10]:
            name = self.raw_data.get('user', {}).get('userName', '')
            return f"AcMoment(@{name} | {str(self.linked)})".encode(errors='replace').decode()
        elif self.tag_rtype == 3:
            name = self.raw_data.get('user', {}).get('userName', '')
            text = self.raw_data.get('moment', {}).get('replaceUbbText', '')
            # text = re.sub('\[emot=acfun,\d+/]', '', thin_string(text))
            text = thin_string(text)
            img_count = self.raw_data.get('discoveryResourceFeedShowImageCount', 0)
            img_count = f"【图x{img_count}】" if img_count > 0 else ""
            link = "" if self.linked is None else f" | {str(self.linked)}"
            return f"AcMoment(@{name}: {img_count}{text}{link})".encode(errors='replace').decode()
        return f"AcMoment()"

    def up(self):
        return self.acer.AcUp(self.raw_data.get('user'))

    def comment(self):
        return self.acer.AcComment(self.rid, self.rtype, self)

    @need_login
    def banana(self, count: int):
        return self.acer.throw_banana(self.rid, self.rtype, count)


class AcMoment:
    cursor = "0"
    limit = 10
    moment_data = list()
    resourceTypes = 0
    rts = {  # 0综合 2视频 3文章 10动态
        'all': 0,
        'video': 2,
        'article': 3,
        'moment': 10
    }

    def __init__(self, acer):
        self.acer = acer

    @need_login
    def set_tab(self, tab: str = 'all'):
        new = self.rts.get(tab, 0)
        if new != self.resourceTypes:
            self.resourceTypes = new
            self.cursor = "0"
            self.moment_data = list()
            return True
        return False

    @need_login
    def feed(self, limit: int = 10, refresh: bool = False):
        if refresh is True:
            self.cursor = "0"
            self.moment_data = list()
        param = {
            "useWebp": False,
            "pcursor": self.cursor,
            "count": limit or self.limit,
            "resourceTypes": self.resourceTypes,
        }
        api_req = self.acer.client.get(apis['follow_feed'], params=param)
        api_data = api_req.json()
        if api_data.get('result') == 0:
            self.cursor = api_data.get('pcursor', self.cursor)
            self.moment_data.extend(api_data.get('feedList', []))
        return [Moment(self.acer, x) for x in api_data.get('feedList', [])]

    def get(self, am_num: [str, int]):
        page_req = self.acer.client.get(f"{routes['moment']}{am_num}")
        page_obj = Bs(page_req.text, 'lxml')
        js_code = page_obj.select_one("#app").find_next_sibling("script").text.strip()
        js_code = re.sub("\;\(function\(\)\{var s\;\(s=document\.currentScript\|\|document\.scripts\[document\.scripts\.length\-1\]\)\.parentNode\.removeChild\(s\)\;\}\(\)\)\;$", '', js_code)
        js_code = re.sub("window\.__INITIAL_STATE__\=", '', js_code)
        page_data = json.loads(js_code).get('moment', {}).get('moment')
        return Moment(self.acer, page_data)




# coding=utf-8
import re
import html
import json
import textwrap
from bs4 import BeautifulSoup as Bs
from acfun.source import routes, apis
from acfun.page.utils import warp_mix_chars, get_page_pagelets
from acfun.saver import ArticleSaver

__author__ = 'dolacmeo'


class AcArticle:
    ac_num = None
    page_obj = None
    page_pagelets = []
    article_data = dict()
    console_width = 80
    is_404 = False

    def __init__(self, acer, ac_num: [str, int], article_data: [dict, None] = None):
        if isinstance(ac_num, str) and ac_num.startswith('ac'):
            ac_num = ac_num[2:]
        self.ac_num = str(ac_num)
        self.referer = f"{routes['article']}{ac_num}"
        self.acer = acer
        if isinstance(article_data, dict):
            self.article_data = article_data
        self.loading()

    def __repr__(self):
        if self.is_404 is True:
            return f"AcArticle([ac{self.ac_num}] 404)"
        title = self.article_data.get('contentTitle', self.article_data.get('title', ""))
        user_name = self.article_data.get('user', {}).get('name', "") or self.article_data.get('user', {}).get('id', "")
        user_txt = "" if len(user_name) == 0 else f" @{user_name}"
        return f"AcArticle([ac{self.ac_num}]{title}{user_txt})".encode(errors='replace').decode()

    @property
    def share_url(self):
        return self.referer

    def loading(self):
        req = self.acer.client.get(routes['article'] + self.ac_num)
        self.page_obj = Bs(req.text, 'lxml')
        if self.page_obj.select_one("#main > script") is None:
            self.is_404 = True
            return
        js_code = self.page_obj.select_one("#main > script").text.strip().split('\n')[0]
        js_head = "window.articleInfo = "
        js_end = ";"
        assert js_code.startswith(js_head) and js_code.endswith(js_end)
        self.article_data = json.loads(js_code[len(js_head):-len(js_end)])
        self.page_pagelets = get_page_pagelets(self.page_obj)

    def saver(self, dest_path=None):
        return ArticleSaver(self.acer, self, dest_path)

    def recommends(self):
        articles = list()
        for item in self.page_obj.select('#main > section.area > .content > .fr > .contribution.weblog-item'):
            this_url = item.select_one('.contb-title a').attrs['href']
            ac_num = this_url[5:]
            data = {
                'title': item.select_one('.contb-title').a.text,
                'shareUrl': routes['share'] + ac_num,
                'viewCount': item.select_one('.contb-count .view-count span.count').text,
                'commentCount': item.select_one('.contb-count .comm-count span.count').text,
                'user': self.article_data.get('user', {})
            }
            articles.append(self.acer.ac_article(ac_num, self.acer, data))
        return articles

    def infos(self):
        if len(self.article_data.keys()) == 0:
            self.loading()
        return self.article_data

    def content(self, num: [int, None] = None):
        show_text = list()
        show_text.extend(textwrap.wrap(self.article_data['title'], 80))
        show_text.extend(textwrap.wrap(self.article_data['description'], 80))
        show_text.append("*" * 80)

        def html2console(data, width=80):
            lines = list()
            page_raw = re.sub('\[emot=acfun,\d+/]', '', data.get('content', ''))
            page_obj = Bs(html.unescape(page_raw), 'lxml')
            for tag in page_obj.select('p,hr'):
                image = tag.select_one('img')
                br = tag.select_one('br')
                if 'hr' in tag.name:
                    lines.append('-' * width)
                elif image is not None:
                    lines.append(f"![ac{self.ac_num}_image]({image.attrs['src']})")
                elif br is not None:
                    lines.append(' ' * width)
                else:
                    this_text = tag.get_text(strip=True)
                    lines.extend(warp_mix_chars(this_text, width))
            return lines

        parts = self.article_data.get('parts', [])
        if len(parts) == 1:
            part_lines = html2console(parts[0])
            show_text.extend(part_lines)
        elif isinstance(num, int) and num in range(len(parts)):
            part_lines = html2console(parts[num])
            show_text.extend(part_lines)
        else:
            for part in parts:
                show_text.append("*" * 80)
                part_lines = html2console(part)
                show_text.extend(part_lines)
        show_text.append("=" * 80)
        return "\n".join(show_text)

    def up(self):
        if len(self.article_data.keys()) == 0:
            self.loading()
        return self.acer.ac_up(self.article_data.get('user', {}), self.acer)

    def comment(self):
        if len(self.article_data.keys()) == 0:
            self.loading()
        return self.acer.AcComment(self.ac_num, 1, self.referer)

    def like(self):
        return self.acer.like(self.ac_num, 3)

    def like_cancel(self):
        return self.acer.like_cancel(self.ac_num, 3)

    def favorite_add(self):
        return self.acer.favourite.add(self.ac_num, 3)

    def favorite_cancel(self):
        return self.acer.favourite.cancel(self.ac_num, 3)

    def banana(self, count: int):
        return self.acer.throw_banana(self.ac_num, 3, count)
